package LectureExamples;

import MDLUtilities.Marquee4Conform;
import PrologDB.DB;
import PrologDB.ErrorReport;
import PrologDB.Table;
import Violett.ClassConform;


public class ConnConform {
    
    public static void main(String... args) {
        // Step 1: Standard marquee processing
        Marquee4Conform mark = new Marquee4Conform(ClassConform.class, ".con.pl", args);
        String inputFileName = mark.getInputFileName();
        
        // Step 2: read database, tables, and create error handler
        DB db = DB.read(inputFileName);
        Table company = db.getTableEH("company");
        Table contract = db.getTableEH("contract");
        Table person = db.getTableEH("contract");
        ErrorReport er = new ErrorReport();
        
        // Evaluate constraints
        // Step 3.1: every person referenced in a contract must correspond to a
        //          person tuple
       contract//.stream()
               .select(t->t.isNotNull("pid"))
               .select(t->!t.joinExists("pid", person, "pid"))
               .forEach(t->er.add("bogus person reference in kid="+ t.get("kid")));
       
       // Step 3.2: every company referenced in a contract must correspond to a
       //           company tuple
       contract//.stream()
               .select(t->t.isNotNull("cid"))
               .select(t->!t.joinExists("cid", company, "cid"))
               .forEach(t->er.add("bogus company reference in kid="+ t.get("kid")));
       
        // Step 3.3: if company has contract, value of contract > 500
        contract//.stream()
                .select(t->t.isNotNull("cid") && t.getInt("value")<=500)
                .forEach(t->er.add("company contract value violated in kid="+ t.get("kid")));
        
        // Step 3.4: if person has contract, value of contract is <100
        contract//.stream()
                .select(t->t.isNotNull("pid") && t.getInt("value")>=100)
                .forEach(t->er.add("person contract value violated in kid="+ t.get("kid")));
        
        // Step 3.5: Contract must be owned by a persor or company but not both
        contract//.stream()
                .select(t-> (t.isNull("pid") && t.isNull("cid")) ||
                            (t.isNotNull("pid") && t.isNotNull("cid") ))
                .forEach(t->er.add("ownership failure in kid="+ t.get("kid")));
        
       // Step 4: output report -- ignore error thrown for this illustration
       try {
           er.printEH(System.out);
       } catch (Exception e) {}
       
       // now, we do the same constraints using built-in MDELite Constraints
       ErrorReport er1 = new ErrorReport();
       
       db.StandardConform(er1, false);
       // does these and more:
       // Step 5.1: every person referenced in a contract must correspond to a
       //      person tuple
       // Table personContracts = contract.select(t->t.isNotNull("pid"));
       // personContracts.isLegit("pid", person, "pid", er1);
       //
       // Step 5.2: every company referenced in a contract must correspond to a
       //      company tuple
       // Table companyContracts = contract.select(t->t.isNotNull("cid"));
       // companyContracts.isLegit("cid", company, "cid", er1);
       
       // Step 5.3: if company has contract, value of contract > 500
       contract.implies(t->t.isNotNull("cid"), t->t.getInt("value")>500, 
               "company contract value violated", er1);
       
       // Step 5.4: if person has contract, value of contract is <100
       contract.implies(t->t.isNotNull("pid"), t->t.getInt("value")<100, 
               "person contract value violated", er1);
       
       // Step 5.5: Contract must be owned by a persor or company but not both
       contract.ifThen(t->(t.isNull("pid")&& t.isNull("cid")) ||
               (t.isNotNull("pid")&& t.isNotNull("cid")), "ownership failure", er1);
       
       // Step 6: output report -- ignore error thrown for this illustration
       try {
           er1.printEH(System.err);
       } catch (Exception e) {}
    }
    
}
