package MDLUtilities;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;

public class DocumentGenerator {

    // this is a (marker, contents) hash map filled in by harvest()
    private static HashMap<String, String> harvested = null;

    /**
     * write out string data to file HtmlFileName
     *
     * @param data -- string of file contents
     * @param HtmlFileName -- name of file to write
     */
    protected static void exhale(String data, String HtmlFileName) {
        try {
            PrintStream out = new PrintStream(HtmlFileName);
            out.println(data);
            out.close();
        } catch (Exception e) {
            throw new RuntimeException("unable to write " + HtmlFileName + "\n" + e.getMessage());
        }
    }

    /**
     * scan sourceCodeFileName for ///Name ... ///Name parentheses whatever is
     * in between associate that with the contents of marker NAME and place into
     * HashMap harvested
     *
     * @param src -- name of source code file to scan
     */
    private static void harvestCode(String... sourceCodeFileNames) {

        harvested = new HashMap<>();
        for (String src : sourceCodeFileNames) {
            if (src.equals("")) {
                continue;
            }

            LineNumberReader br;
            String markerName = null;
            String accum = null;
            try {
                br = new LineNumberReader(new InputStreamReader(new FileInputStream(new File(src))));
                String line;
                while ((line = br.readLine()) != null) {
                    if (line.startsWith("///")) {
                        // first time through, get marker name and init accum
                        if (markerName == null) {
                            markerName = line.replace("///", "").trim();
                            accum = "";
                            continue;
                        } else {
                            // second time through erase marker name and save accum
                            harvested.put(markerName, accum);
                            markerName = null;
                            accum = null;
                            continue;
                        }
                    }
                    if (markerName != null) {
                        accum = accum + line + "<br>";
                    }
                }
                br.close();
            } catch (Exception e) {
                throw new RuntimeException(e.getMessage());
            }
            if (markerName != null) {
                throw new RuntimeException("in " + src + ": run-away marker ///" + markerName + " -- no end!");
            }
        }
    }

    /**
     * return the String that is the content of the template file
     *
     * @param htmFile -- path to template file
     * @return -- string that is the contents of the Ht
     */
    protected static String inhale(String htmFile) {
        LineNumberReader reader = null;
        String accum = "";
        String line;
        try {
            reader = new LineNumberReader(new InputStreamReader(new FileInputStream(new File(htmFile))));
            while ((line = reader.readLine()) != null) {
                String[] toks = line.split(":::");
                boolean odd = true;
                for (String s : toks) {
                    if (odd) {
                        accum += s + " ";
                    } else {
                        String contents = readFile(s);
                        accum = accum.trim() + contents + " ";
                    }
                    odd = !odd;
                }
            }
            reader.close();
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        }
        return accum;
    }

    /**
     * create .html file outputHtml given a .htm BuildDocTest and the
     * sourceCodeFile (typically .java) from which to extract code. A marker is
     * " ---NAME--- " in an .htm template file A marker is "///NAME " in a
     * sourcecode file
     *
     * @param sourceCodeFileNames -- file that has named begin+end markers --
     * doesn't have to be a .java file
     * @param templateFile --- .htm file with named markers (where source code
     * is to be inserted)
     * @param outputHtml -- output .html file
     */
    public static void main(String templateFile, String outputHtml, String... sourceCodeFileNames) {
        if (!templateFile.endsWith(".htm")) {
            throw new RuntimeException("template file \'" + templateFile + "' does not end in .htm");
        }
        if (!(outputHtml.endsWith(".html") || outputHtml.endsWith(".htm"))) {
            throw new RuntimeException("output file \'" + outputHtml + "' does not end in .htm/.html");
        }
        harvestCode(sourceCodeFileNames); // determines --- for labels---
        String htmlFile = inhale(templateFile); // fills in results
        for (Map.Entry<String, String> e : harvested.entrySet()) {
            String key = "---" + e.getKey() + "---";
            if (htmlFile.contains(key)) {
                htmlFile = htmlFile.replace(key, e.getValue());
            }
        }
        exhale(htmlFile, outputHtml);
    }

    private static String readFile(String filename) {
        String accum = "";
        String line;
        try {
            LineNumberReader br = new LineNumberReader(new InputStreamReader(new FileInputStream(new File(filename))));
            while ((line = br.readLine()) != null) {
                accum += line + "\n";
            }
            br.close();
        } catch (Exception e) {
            throw new RuntimeException("unable to read " + filename + "\n" + e.getMessage());
        }
        return accum.trim();
    }

}
