package MDLUtilities;

import static MDLUtilities.Error.fileNoExist;
import java.io.FileNotFoundException;
import java.io.PrintStream;

/** standard MDELite Marquee processing */
public class Marquee1In_1Out extends MarqueeCommon {

    String outputPattern1;

    /** standard usage:
        Marquee1In_1Out mark = new Marquee1In_1Out(THIS.class, ".a.b", ".c.d", args);
        String inputFileName = mark.getInputFileName();
        String outputFileName = mark.getOutputFileName();
        String appName = mark.getAppName();
     * @param cls -- class object of MDELite application
     * @param inputFilePattern -- one input pattern
     * @param outputFilePattern -- one output pattern
     * @param args -- command-line arguments from main(args)
     */
    public Marquee1In_1Out(Class cls, String inputFilePattern, String outputFilePattern, String args[]) {
        super(cls, inputFilePattern, args);
        this.outputPattern1 = outputFilePattern;
        if (args.length==0)
            displayMarquee(null);
    }

    /** create a marquee -- I think to passify a whiney compiler */
    public Marquee1In_1Out() {
    }

    /**
     * display generic marquee
     * @param msg -- if there is a special error, provide it here; "" otherwise
     */
    public void displayMarquee(String msg) {
        if (msg != null) {
            System.out.format("\n---> %s\n", msg);
        }
        System.out.format("\nUsage: %s X%s [X%s]\n", cls.getName(), inputPattern1, outputPattern1);
        System.out.format("       translate X%s to X%s\n", inputPattern1, outputPattern1);
        System.exit(0);
    }

    /**
     * @return string of input file name (the first argument)
     */
    @Override
    public String getInputFileName() {
        int len = args.length;
        if (len == 0) {
            displayMarquee(null);
        }
        if (!args[0].contains(".")) //NEW
            return args[0]+inputPattern1;
        if (!args[0].endsWith(inputPattern1)) {
            displayMarquee(args[0] + " submitted as input");
        }
        return args[0];
    }

    /**
     * @return -- output file name (second argument)
     */
    public String getOutputFileName() {
        if (args.length==2) 
            return args[1];
        String inputFileName = getInputFileName();
        if (inputFileName.contains("/")) {
            int x = inputFileName.lastIndexOf('/');
            String y= inputFileName.substring(x+1).replace(inputPattern1, "")+outputPattern1;
            return y;
        }
        if (inputFileName.endsWith(inputPattern1))
            return inputFileName.replace(inputPattern1,outputPattern1);
        return inputFileName+outputPattern1;
//        if (!args[0].contains(".")) //NEW
//            return args[0]+outputPattern1;
//        return args[0].replace(inputPattern1,outputPattern1); 
////        return getAppName()+outputPattern1;
    }

    /**
     * @param i -- index of command-line argument to return
     * @return argument # i
     */
    public String getArgument(int i) {
        switch (i) {
            case 0:
                return getInputFileName();
            case 1:
                return getOutputFileName();
            default:
                if (i < 0 || i >= args.length) {
                    displayMarquee("input index " + i + " requested and doesn't exist");
                } else {
                    return args[i];
                }
        }
        return null; // never reaches here; added to pacify whiney compiler
    }

    /**
     * @return PrintStream of outputFile
     */
    public PrintStream getOutputPrintStream() {
        PrintStream out = null;
        String outFile = getOutputFileName();
        try {
            out = new PrintStream(outFile);
        } catch (FileNotFoundException e) {
            throw Error.toss(fileNoExist, outFile);
        }
        return out;
    }
    
    /** output file names are dir/dir/dir/X.schemaname.pl
     * @return -- name dir/dir/dir/X of output application
     */
    public String getLongAppName() {
        String shorten = getOutputFileName().replace(outputPattern1, "");
        return shorten;
    }
}
