package MDLUtilities;

import static MDLUtilities.Error.fileNoExist;
import java.io.FileNotFoundException;
import java.io.PrintStream;

/** standard MDELite Marquee processing */
public class MarqueeMulti1In_1Out {

    String inputPatterns[], outputPattern, args[], AppName;
    Class cls;

    /** standard usage:
        MarqueeMulti1In_1Out mark = new MarqueeMulti1In_1Out(THIS.class, inputPatterns[], outputPattern, args[]);
        String inputFileName = mark.getInputFileName();
        String outputFileName = mark.getOutputFileName();
        String appName = mark.getAppName();
     * @param cls -- class object of MDELite application
     * @param inputPatterns -- array of 2 or more input patterns
     * @param outputPattern -- one output pattern
     * @param args -- command-line arguments from main(args)
     */
    public MarqueeMulti1In_1Out(Class cls, String[] inputPatterns, String outputPattern, String args[]) {
        this.cls = cls;
        this.args = args;
        this.inputPatterns = inputPatterns;
        this.outputPattern = outputPattern;
        if (args.length==0 || inputPatterns.length<2 || outputPattern.equals(""))
            displayMarquee(null);
    }

    /** create a marquee -- I think to passify a whiney compiler */
    public MarqueeMulti1In_1Out() {
    }

    /**
     * display generic marquee
     * @param msg -- if there is a special error, provide it here; "" otherwise
     */
    public void displayMarquee(String msg) {
        if (msg != null) {
            System.out.format("\n---> %s\n", msg);
        }
        String s ="";
        String comma = "";
        for (String k : inputPatterns) {
            s = s + comma + "X"+k;
            comma = " | ";
        }
        System.out.format("\nUsage: %s %s [X%s]\n", cls.getName(), s, outputPattern);
        System.out.format("       translate (input) to X%s\n", outputPattern);
        System.exit(0);
    }

    /**
     * @return string of input file name (the first argument)
     */
    public String getInputFileName() {
        int len = args.length;
        if (len == 0) {
            displayMarquee(null);
        }
        boolean ok = false;
        AppName = null;
        for (String s: inputPatterns) {
            if (args[0].endsWith(s)) {
                AppName = args[0].replace(s,"");
                ok = true;
                break;
            }
        }
        if (!ok) {
            displayMarquee(args[0] + " submitted as input");
        }
        return args[0];
    }

    /**
     * argument 0 input file
     * argument 1 output file
     * @return -- output file name (second argument)
     */
    public String getOutputFileName() {
        if (args.length==2) 
            return args[1];
        return getAppName()+outputPattern;
    }
    
    public String getAppName() {
        if (AppName==null)
            displayMarquee("didn't call getInputFileName first");
        return AppName;
    }

    /**
     * @return PrintStream of outputFile
     */
    public PrintStream getOutputPrintStream() {
        PrintStream out = null;
        String outFile = getOutputFileName();
        try {
            out = new PrintStream(outFile);
        } catch (FileNotFoundException e) {
            throw Error.toss(fileNoExist, outFile);
        }
        return out;
    }

}
