package MDLUtilities;


import PrologDB.DB;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.PrintStream;

/**
 * RunningBear is a pure Java Model-to-Text tool to replace Velocity
 */
public abstract class RunningBear {

    /**
     * db is the current MDELite7 database db = DB.read(database);
     */
    public static DB db;
    public static String databaseInputFile;
    public static PrintStream out = System.out;

    /**
     * opens database and creates PrintStream for RunningBear output
     *
     * @param mark -- a Marquee object
     * @param args -- command-line arguments from main(args)
     */
    public static void RBSetupDir(MarqueeCommon mark, String[] args) {

        // Step 2: open database, if necessary
        databaseInputFile = mark.getInputFileName();
        if (databaseInputFile != null && databaseInputFile.equals("null")) {
            databaseInputFile = null;
        }
    }

    /**
     * opens database and creates PrintStream for RunningBear output
     *
     * @param mark -- a Marquee object
     * @param args -- command-line arguments from main(args)
     */
    public static void RBSetup(Marquee1In_1Out mark, String[] args) throws RuntimeException {

        // Step 2: open database, if necessary
        databaseInputFile = mark.getInputFileName();
        if (databaseInputFile != null && databaseInputFile.equals("null")) {
            databaseInputFile = null;
        }

        if (databaseInputFile != null) {
            db = DB.read(databaseInputFile);
        }

        // Step 3: open output file, if necessary
        String ofn = mark.getOutputFileName();
        openOut(ofn);
    }

    /**
     * @return the name of the current database file
     */
    public static String getDatabaseInputFile() {
        return databaseInputFile;
    }

    public static DB getDB() {
        return db;
    }

    /**
     * printline(format, arguments)
     * @param fmt -- format
     * @param args -- argument list
     */
    public static void l(String fmt, Object... args) {
        out.format(fmt + "\n", args);
    }

    /**
     * print empty line
     */
    public static void l() {
        out.format("\n");
    }

    /**
     * print n empty lines
     *
     * @param n -- number of empty lines to print
     */
    public static void l(int n) {
        for (int i = 1; i <= n; i++) {
            l();
        }
    }

    /**
     * print(format, arguments) -- no line eject
     *
     * @param fmt -- format
     * @param args -- argument list
     */
    public static void p(String fmt, Object... args) {
        out.format(fmt, args);
    }

    /**
     * send all further output to outputFile closeOut the current file (unless
     * it is System.out)
     *
     * @param outputFile -- the name of the output file for RunningBear
     */
    public static void openOut(String outputFile) {
        if (out != System.out) {
            closeOut();
        }
        try {
            out = new PrintStream(outputFile);
        } catch (Exception e) {
            throw Error.toss(Error.ioerror, outputFile, e.getMessage());
        }
    }

    /**
     * send all further output to outputFile closeOut the current file (unless
     * it is System.out)
     *
     * @param o -- new output stream for RB
     */
    public static void openOut(PrintStream o) {
//        if (o != System.out) {
//            closeOut();
//        }
        out = o;
    }

    /**
     * closeOut the current output file (unless it is System.out)
     */
    public static void closeOut() {
        if (out != System.out) {
            out.close();
            out = System.out;
        }
    }

    /**
     * @return output printstream
     */
    public static PrintStream getOut() {
        return out;
    }
    
    public static void AppendFile(String filename) {
        File in = new File(filename);
        if (!in.exists()) {
            throw Error.toss(Error.fileNoExist, filename);
        }
        LineNumberReader br;
        String line;
        try {
            // Step 1: Read infile by line
            br = new LineNumberReader(new InputStreamReader(new FileInputStream(in)));

            while ((line = br.readLine()) != null) {
                out.println(line);
            }
            br.close();
        } catch (IOException e) {
            throw Error.toss(Error.ioerror, filename, e.getMessage());
        }
    }
    
     public static void Append(String[] lines) {
        for (String ln : lines) {
            out.println(ln);
        }
    }
}
