package Parsing.GeneralPrimitives.Patterns;

import MDLUtilities.ParsE;
import Parsing.GeneralPrimitives.Token;
import Parsing.LineParsers.LineToParse;
import java.util.List;

/** pattern is a sequence of tokens to parse */
public abstract class Pattern extends Token {

    protected Token[] sequence = null; // filled in by subclasses
    
    protected Pattern() { } 
    
    /** Standard constructor
     * @param array -- sequence of tokens to be parsed
     */
    public Pattern(Token[] array) {
        super();
        this.sequence = array;
        if (array.length == 0) {
            throw ParsE.toss(ParsE.parseError, -1);
        }
    }

    /** Standard constructor with line to parse
     *
     * @param l -- line to parse
     */
    public Pattern(LineToParse l) {
        super(l);
    }

    // I added this method to assign the sequence variable
    // and avoid static initialization, which in recursive
    // patterns surprisingly doesn't work
    abstract public Token[] getSequence();
    
    private Token first;

    @Override
    public boolean canParse() {     
        first = getSequence()[0].klone(toBeParsed);
        return first.canParse();
    }

    @Override
    public void consumeToken() {
        first.consumeToken();
    }

    @Override
    public String getEmsg() {
        Token[] s = getSequence();
        first = s[0].klone(toBeParsed);
        return first.getEmsg();
    }


    @Override
    public void parse() {
        Token[] defn = getSequence();
        for (Token t : defn) {
            t.setLineToParse(toBeParsed);
            if (t.canParse()) {
                t.parse();
            } else {
                if (toBeParsed.atEnd())
                    throw ParsE.toss(ParsE.streamEndReached, toBeParsed.getLineNo(), t.getEmsg());
                else 
                    throw ParsE.toss(ParsE.parseError, toBeParsed.getLineNo(), 
                            String.format("'%s' expected at >>%s",t.getEmsg(), toBeParsed.toParse));
            }
        }
    }
   
}
