package Parsing.GeneralPrimitives;

import Parsing.LineParsers.LineToParse;

/** a token is a primitive unit of parsing */
public abstract class Token {
    
    protected LineToParse toBeParsed;  // String to be parsed
    
    /** Standard constructor */
    protected Token() { }
    
    /** Standard constructor with line to parse
     * @param l -- line to parse
     */
    protected Token(LineToParse l) {
        toBeParsed = l;
    }
    
    /**
     * only to be called in Pattern and Choose1 templates to set the current
     * line on the given pattern
     * set line to parse (l)
     * @param l -- the LineToParse object
     */
    public void setLineToParse(LineToParse l) {
       toBeParsed = l;
    }
    
    /**
     * can this token be parsed?  is the next character
     * of toBeParsed consistent with the requested parsing task?
     * no changes to LineToParse are made
     * @return true if the next character to parse is consistent with requested parsing task
     */
    public abstract boolean canParse();
    
    /** assuming canParse has been called, consume the next token; 
     * parsed token is stored in LinetoParse object */
    public abstract void consumeToken();

    /** actually parse toBeParsed given requested parsing task */
    public abstract void parse();
    
    /** return copy of 'this'
     * @param l -- line to parse
     * @return  -- token object to parse this line
     */
    public abstract Token klone(LineToParse l);
    
    /** 
     * @return  -- string message to report about token in an error
     */
    public abstract String getEmsg();
    
    /**
     * basically this returns the last parsed token.  Remember some tokens
     * have a string that is fixed, like "dbase".  Others, like Name and
     * NumBer are literally computed.  When they are parsed, they are stored
     * in this.toBeParsed.token -- the last token that was parsed.
     * @return last token to be parsed, which usually is the string of 
     * the last token consumed
     */
    public String getTokenString() {
        return this.toBeParsed.token;
    }
}
