package PrologDB;

import MDLUtilities.Error;
import java.io.PrintStream;

/**
 * ErrorReport is a standard object to collect errors in evaluating database
 * constraints; it remembers a list of errors, and when its report is printed,
 * it throws Error if errors are reported
 *
 */
public class ErrorReport {

    /**
     * this is the string accumulator of errors; initially it is empty.
     */
    private String errorReport = "";
    private int errorCount = 0;

    /**
     * returns a fresh ErrorReport object
     */
    public ErrorReport() {
    }

    /**
     * add errorMsg to report string
     *
     * @param errorMsg -- errorMsg to report string
     */
    public void add(String errorMsg) {
        errorReport += errorMsg.replace("\n", "") + "\n";
        errorCount++;
    }

    /**
     * add error message (fmt,args) to report string
     *
     * @param fmt -- format string
     * @param args -- arguments of format string
     */
    public void add(String fmt, Object... args) {
        errorReport += String.format(fmt + "\n", args);
        errorCount++;
    }

    /**
     * for debugging -- add barrier in report without adding an error
     */
    public void addBarrier() {
        errorReport += "--------\n";
    }

    /**
     * @param Banner -- to print on errors
     * @param out -- where to report errors
     */
    public void printEH(String Banner, PrintStream out) {
        if (errorCount != 0) {
            out.format("%s\n\n", Banner);
            out.println(errorReport);
            out.flush();
            throw Error.toss(Error.errorReport, errorCount + "");
        }
    }
    
    public void printEH() {
        printEH(System.out);
    }
    
    
    public boolean print() { 
        return print(System.out);
    }

    /**
     * print report to PrintStream out; if no errors are reported, print
     * returns; otherwise a Error is thrown
     *
     * @param out -- printstream to output report
     */
    public void printEH(PrintStream out) {
        if (print(out)) {
            throw Error.toss(Error.errorReport, errorCount + "");
        }
    }

    /**
     * print report to PrintStream out; if no errors are reported, print
     * returns; otherwise a Error is thrown
     *
     * @param out -- printstream to output report
     * @return true if there are errors
     */
    public boolean print(PrintStream out) {
        if (errorCount == 0) {
            return false;
        }
        out.println(errorReport);
        out.flush();
        return true;
    }

    /**
     * @return status if there are errors
     */
    public boolean hasErrors() {
        return errorCount > 0;
    }

    /**
     * @return number of errors
     */
    public int errorCount() {
        return errorCount;
    }

    /**
     * for debugging purposes
     */
    public String toString() {
        return errorReport;
    }

    /**
     * makes ErrorReporters reusable by re-initializing their state
     */
    public void clear() {
        errorReport = "";
        errorCount = 0;
    }
}
