package Violett;

import MDLUtilities.Marquee4Conform;
import PrologDB.DB;
import PrologDB.ErrorReport;
import PrologDB.Table;
import PrologDB.Tuple;
import java.util.function.Predicate;

/** conformance program for Violet .vpl.pl databases */
public class ClassConform {

    /** conformance program for Violet .vpl.pl databases
     * @param args X.vpl.pl
     */
    public static void main(String... args) {
        String expl;
        
        // Step 1: Standard marquee processing
        Marquee4Conform mark = new Marquee4Conform(ClassConform.class, ".vpl.pl", args);
        String inputFileName = mark.getInputFileName();

        // Step 2: read database and create error handler
        DB db = DB.read(inputFileName);
        check(db);
    }

    public static void check(DB db) {
        String expl;
        Table vBox = db.getTableEH("vBox");
        Table vAssociation = db.getTableEH("vAssociation");
        ErrorReport er = new ErrorReport();
        
        db.StandardConform(er, false);
        // checks
//        // C1: all vBox ids are unique
//        // C2: all vAssociation ids are unique
//        vBox.isUnique("id", er);
//        vAssociation.isUnique("id", er);
//        
//        // C3: All cid1 references are legit
//        // C4: All cid2 references are legit
//        vAssociation.isLegit("cid1", vBox, "id", er);
//        vAssociation.isLegit("cid2", vBox, "id", er);
        
        // vBox constraints
        // % type = c(class),i(nterface),n(ote).  packages are skipped   
        // table(vBox,[id,type,"name","fields","methods",x,y]).
        // C5: if type = i, then fields are empty
        // C6: if type = n, then fields and methods are empty
        // C6': if abst= true then type = c
        Predicate<Tuple> typeisi = r -> r.is("type", "i");
        Predicate<Tuple> typeisn = r -> r.is("type", "n");
        Predicate<Tuple> typeisc = r -> r.is("type", "c");
        Predicate<Tuple> isAbstract = r -> r.is("abst", "true");
        Predicate<Tuple> fieldsAreEmpty = r -> r.is("fields", "");
        Predicate<Tuple> methodsAreEmpty = r -> r.is("methods", "");
        expl = "interfaces should have no fields";
        vBox.implies(typeisi, fieldsAreEmpty, expl, er);
        expl = "notes should have no fields or methods";
        vBox.implies(typeisn, fieldsAreEmpty.and(methodsAreEmpty), expl, er);
        expl = "abst='true'; only classes can be abstract";
        vBox.implies(isAbstract, typeisc, expl, er);
        
        // % type1,type2 = c(lass) or i(nterface)
        //% bentStyle = "-"(solid) or "-.-"
        //% lineStyle = ""(solid) or "DOTTED"
        //% arrow1,2 = V, TRIANGLE, DIAMOND, BLACK_DIAMOND
        //% bentStyle = "", HV, VH, HVH, VHV
        //table(vAssociation,[id,cid1,type1,"role1","arrow1",cid2,type2,"role2","arrow2","bentStyle","lineStyle","middleLabel"]).
        // C7,8: if arrow1==TRIANGLE implies arrow2isEmpty and role1isEmpty and role2isEmpty (symmetrical)
        Predicate<Tuple> a1isTriangle = t -> t.is("arrow1", "TRIANGLE");
        Predicate<Tuple> a2isTriangle = t -> t.is("arrow2", "TRIANGLE");
        Predicate<Tuple> a1isEmpty = t -> t.is("arrow1", "");
        Predicate<Tuple> a2isEmpty = t -> t.is("arrow2", "");
        Predicate<Tuple> r1isEmpty = t -> t.is("role1", "");
        Predicate<Tuple> r2isEmpty = t -> t.is("role2", "");
        expl = "arrow1=TRIANGLE implies (arrow2 is empty and role1 is empty and role2 is empty)";
        vAssociation.implies(a1isTriangle, a2isEmpty.and(r1isEmpty.and(r2isEmpty)), expl, er);
        vAssociation.implies(a2isTriangle, a1isEmpty.and(r1isEmpty.and(r2isEmpty)), expl, er);
        
        // C11 if arrow1==TRIANGLE and type1=i and type2=c, then lineStyle== dashed, symmetrically
        Predicate<Tuple> i1andc2 = r -> r.is("type1", "i") && r.is("type2", "c");
        Predicate<Tuple> i2andc1 = r -> r.is("type2", "i") && r.is("type1", "c");
        Predicate<Tuple> isDashed = r -> r.is("lineStyle", "DOTTED");
        expl = "linestyle should be dashed)";
        vAssociation.implies(a1isTriangle.and(i1andc2).or(a2isTriangle.and(i2andc1)),
                isDashed, expl, er);

        // See if there are extends cycles
        Predicate<Tuple> candc = y -> y.is("type1", "c") && y.is("type2", "c");
        Table ct = Table.makeCycleTable();
        vAssociation.stream().filter(a1isTriangle.and(candc)).forEach(t -> addTuple(t, ct, true));
        vAssociation.stream().filter(a2isTriangle.and(candc)).forEach(t -> addTuple(t, ct, false));
        ct.cycleCheck(er);
        
        // Finally, see if there are implements cycles
        Table ct2 = Table.makeCycleTable();
        vAssociation.stream().filter(a1isTriangle.and(candc.negate())).forEach(t -> addTuple(t, ct2, true));
        vAssociation.stream().filter(a2isTriangle.and(candc.negate())).forEach(t -> addTuple(t, ct2, false));
        ct2.cycleCheck(er);
        
        // End report all errors
        er.printEH(System.out);
    }
                
    private static void addTuple(Tuple t, Table ct, boolean direction) {
        Tuple cycle = new Tuple(ct);
        if (direction) {
            cycle.setValues(t.get("cid1"), t.get("cid2"));
        } else {
            cycle.setValues(t.get("cid2"), t.get("cid1"));
        }
        ct.add(cycle);
    }
}
