package DML.PrologDB;

import java.util.HashMap;
import java.util.function.Predicate;
import org.junit.Test;
///Check
import PrologDB.*;

public class DocConstraintExamplesTest  {

    public static void main(String... args) {
        // Step 1: standard command-line processing
        MDLUtilities.Marquee4Conform mark = new MDLUtilities.Marquee4Conform(DocConstraintExamplesTest.class, ".sc.pl", args);
        String inputFileName = mark.getInputFileName();
        String AppName = mark.getAppName();

        // Step 2: open database to be validated + get needed tables
        DB db = DB.read(inputFileName);
        Table course = db.getTableEH("course");
        Table student = db.getTableEH("student");
        Table takes = db.getTableEH("takes");
        ErrorReport er = new ErrorReport();

        // Step 3: now perform database checks
        //...
        // Step 4: finish by reporting collected errors
        er.printEH(System.out);
    }
///Check

    static final String testdata = "test/DML/PrologDB/TestData/";
    static final String correct = "test/DML/PrologDB/Correct/";
    static final String errorfile = "trash/error.txt";

    public DocConstraintExamplesTest() {
    }

    @Test
    public void nonNullFieldTest() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            NonNull();
        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "nonnull.txt", false);
    }

    void NonNull() {
        DB db = DB.read(testdata + "/StudentCourse.sc.pl");
        Table takes = db.getTableEH("takes");
        ErrorReport er = new ErrorReport();
///NonNull
        // Step 1: MDELite built-in support
        takes.isNotNull("cid",er);
        takes.isNotNull("sid",er);
        
        // Step 2: manual way
        Predicate<Tuple> cidIsNull = t -> t.isNull("cid");
        takes.select(cidIsNull).error(er,"takes(%s...) has cid=null",t->t.getId());
        takes.ifThen(t->t.isNull("sid"), "sid=null", er); // shorter way
///NonNull

        // finish
        er.printEH(System.out);
    }


    @Test
    public void existenceTest() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            Existence();
        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "existence.txt", true);
    }

    void Existence() {
        DB db = DB.read(testdata + "/StudentCourse.sc.pl");
        Table takes = db.getTableEH("takes");
        Table course = db.getTableEH("course");
        ErrorReport er = new ErrorReport();
///Existence
        // Step 1: MDELite built-in support
        takes.isLegit("cid", course, "cid", er);
        
        // Step 2: Manual: compute a table of offending tuples
        takes.select(t->t.isNotNull("cid")).leftAntiSemiJoin("cid", course, "cid")
             .error(er, "takes(%s,..) has invalid cid value (%s)", t->t.getId(),t->t.get("cid"));
        
        // Step 3: really low-level
        takes.select(t -> t.isNotNull("cid") && (!course.exists("cid", t.get("cid"))))
                .forEach(t -> {
                    er.add("takes(%s,..) has invalid cid value (%s)",
                            t.get("tid"), t.get("cid"));
                });
///Existence
        // finish
        try {
            er.printEH(System.out);
        } catch (Exception ex) {
        }

    }

    @Test
    public void uniqueIDTest() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            UniqueID();
        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "unique.txt", false);
    }

    void UniqueID() {
        DB db = DB.read(testdata + "/StudentCourse.sc.pl");
        Table course = db.getTableEH("course");
        ErrorReport er = new ErrorReport();
///UniqueID
        // Step 1: simplest way uses MDELite built-in support
        course.uniqueId(er);
        
        // Step 2: a more general way -- that works for any column
        String column = course.getIDName();
        course.project(column)
              .duplicates()
              .error(er,"course table has multiple tuples with id='%s'",t->t.getId());
///UniqueID
        // finish
        er.printEH(System.out);
    }

    @Test
    public void uniqueNameTest() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            UniqueName();
        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "uniqueName.txt", false);
    }

    void UniqueName() {
        DB db = DB.read(testdata + "/StudentCourse.sc.pl");
        Table student = db.getTableEH("student");
        ErrorReport er = new ErrorReport();
///UniqueName
        // Step 1: MDELite built-in support
        student.isUnique("name", er);
        
        // Step 2: more general way
        student.project("name")
               .duplicates()
               .error(er,"multiple tuples in student have name='%s'",t->t.get("name"));
///UniqueName
        // finally
        er.printEH(System.out);
    }

    @Test
    public void uniqueComboTest() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            UniqueCombo();
        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "uniqueCombo.txt", false);
    }

    void UniqueCombo() {
        DB db = DB.read(testdata + "/StudentCourse.sc.pl");
        Table takes = db.getTableEH("takes");
        ErrorReport er = new ErrorReport();
///UniqueCombo
        // Step 1: create table with only cid and sid columns
        takes.project("cid","sid")
             .duplicates()
             .error(er,"tuples in takes with duplicate (%s,%s) values",t->t.get("cid"),t->t.get("sid"));
///UniqueCombo
        // finally
        er.printEH(System.out);
    }



    @Test
    public void attendanceTest() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            attendance();
        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "attendance.txt", false);
    }

    void attendance() {
        DB db = DB.read(testdata + "/StudentCourse.sc.pl");
        Table course = db.getTableEH("course");
        Table takes = db.getTableEH("takes");
        ErrorReport er = new ErrorReport();
///attendance
        // check constraint
        course.select(t -> t.is("name", "compilers") || t.is("name", "databases"))
              .select(t -> t.join("cid", takes, "cid").count() < 2)
              .error(er,"%s course does not have enough students", t->t.get("name"));
///attendance
        // finally
        er.printEH(System.out);
    }

    @Test
    public void jConstraintTest() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            jConstraint();
        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "jConstraint.txt", false);
    }

    void jConstraint() {
        DB db = DB.read(testdata + "/StudentCourse.sc.pl");
        Table student = db.getTableEH("student");
        Table takes = db.getTableEH("takes");
        Table course = db.getTableEH("course");
        ErrorReport er = new ErrorReport();
///jConstraint
        // check constraint
        student.forEach(s -> {
            Table shortTakes = s.rightSemiJoin("sid", takes, "sid");
            Table joined = shortTakes.join("cid", course, "cid");
            if (joined.count() == 0) {
                er.add("student %s is not taking a course", s.get("name"));
            }
        });
///jConstraint
        // finally
        er.printEH(System.out);
    }

    @Test
    public void jConstraint2Test() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            jConstraint2();
        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "jConstraint.txt", false);
    }

    void jConstraint2() {
        DB db = DB.read(testdata + "/StudentCourse.sc.pl");
        ErrorReport er = new ErrorReport();
        
///jConstraint2
        Table student = db.getTableEH("student");
        Table takes = db.getTableEH("takes");
        Table course = db.getTableEH("course");
        Table tc = takes.join("cid", course, "cid");

        student.select(s -> s.rightSemiJoin("sid", tc, "takes.sid").count() == 0)
                .error(er,"student %s is not taking a course", s->s.get("name"));
///jConstraint2
        // finally
        er.printEH(System.out);
    }
    
    @Test
    public void jConstraint3Test() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            jConstraint2();
        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "jConstraint.txt", false);
    }

    void jConstraint3() {
        DB db = DB.read(testdata + "/StudentCourse.sc.pl");
        ErrorReport er = new ErrorReport();
        
///jConstraint3
        Table student = db.getTableEH("student");
        Table takes = db.getTableEH("takes");

        student.select(s -> s.rightSemiJoin("sid", takes, "sid").count() == 0)
                .error(er,"student %s is not taking a course", s->s.get("name"));
///jConstraint3
        // finally
        er.printEH(System.out);
    }

    @Test
    public void joinTest() {
        RegTest.Utility.redirectStdOut(errorfile);
        try {
            HashMap<String, Integer> collection = new HashMap<>();
            DB db = DB.read(testdata + "/dogOwner.do.pl");
            ErrorReport er = new ErrorReport();

            Table dog = db.getTableEH("dog");
            Table dogc = dog.copyForSelfJoins("dogc");
            Table dxdc = dog.join("name", dogc, "name");
            for (Tuple t : dxdc.tuples()) {
                String key = t.get("dog.name");
                Integer i = collection.get(key);
                i = (i == null) ? 1 : i + 1;
                collection.put(key, i);
            }
            for (String k : collection.keySet()) {
                Integer i = collection.get(k);
                if (i != 1) {
                    er.add("%s has %d replicas", k, i / 2);
                }
            }
            System.out.println("dogs with the same name");
            er.printEH(System.out);

        } catch (Exception e) {
        }
        RegTest.Utility.validate(errorfile, correct + "jointest.txt", false);
    }
}
