package PrologDB;

import Aggregation.Count;
import Aggregation.DoubleAgg;
import Aggregation.IntSum;
import RegTest.Utility;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;


public class AggregationTest {
    static String correct = "test/PrologDB/Correct/";
    static String outputFile = "trash/error.txt";
    
    static DB db;
    static Table employees, parts, customers, orders, odetails, zipcodes;
    
    public AggregationTest() {
    }
    
    @BeforeClass
    static public void setUp() {
        db = DB.read(correct+"ex.mailorder.pl");
        employees = db.getTable("employees");
        parts = db.getTable("parts");
        customers = db.getTable("customers"); 
        orders = db.getTable("orders");
        odetails = db.getTable("odetails");
        zipcodes = db.getTable("zipcodes");
    }
    
    @Before
    public void init() {
        Utility.init();
        Utility.redirectStdOut(outputFile);
    }

    @Test
    public void wichita() {
        // how many employees are from wichita
///wichita   
        String[] fields = {"zipcodes.city"};
        employees.join("zip", zipcodes.select(z->z.is("city","wichita")), "zip")
                 .groupBy(fields,new Count("cnt"))
                 .print();
        // or
        int i = zipcodes.select(z->z.is("city","wichita")).rightSemiJoin("zip",employees,"zip").count();
        System.out.println("count = " + i);
///wichita  
        Utility.validate(outputFile, correct+"wichita.txt", false);
    }

    
    @Test
    public void land() {
        // ignoring versions, how many Land before times have been sold?
        // allegory example with aggregation
///land
        Table t = parts.select(p->p.get("pname").startsWith("land before time"))
                       .rightSemiJoin("pno", odetails, "pno")
                       .aggregate(new IntSum("sum","qty"));
        t.print();
        System.out.println("Number of LBT sold = " + t.getFirst().get("sum"));
///land
        Utility.validate(outputFile, correct+"land.txt", false);
    }
    
    @Test
    public void whoSoldWhat() {
        // for each employee, how many parts has he sold?
///whoSoldWhat
        Table t = employees.join("eno", orders, "eno").join("orders.ono", odetails, "ono");
        String[] fields = {"employees.eno","employees.ename"};
        Table r = t.groupBy(fields, new IntSum("Nsold","odetails.qty"));
        r.print();
///whoSoldWhat
        Utility.validate(outputFile, correct+"whoSoldWhat.txt", false);
    }

    
    @Test
    public void onhand() {
        // for each part, how much money is in our warehouse?
///onhand
        DoubleAgg da = new DoubleAgg("money",(i,tpl) -> i + tpl.getInt("qoh")*tpl.getDouble("price"));
        parts.aggregate(da).print();
///onhand
        Utility.validate(outputFile, correct+"onhand.txt", false);
    }

    
    @Test
    public void costPerOrder() {
        // what is the cost per order?//costPerOrder
///costPerOrder
        Table j = orders.join("ono",odetails,"ono")
                        .join("odetails.pno",parts,"pno");
        String[] fields = {"odetails.ono"};
        Table r = j.groupBy(fields, 
                new DoubleAgg("cost", 
                        (d,t)-> d + t.getInt("odetails.qty")*t.getDouble("parts.price") ));
        r.print();
///costPerOrder
        Utility.validate(outputFile, correct+"costPerOrder.txt", false);
    }

    
    @Test
    public void DirtyHarry() {
        // what customers ordered dirty harry or dr zhivago?
///dirtyHarry
        parts.select(p->p.is("pname","dirty harry","dr. zhivago"))
             .rightSemiJoin("pno", odetails, "pno")
             .rightSemiJoin("ono", orders, "ono")
             .rightSemiJoin("cno", customers, "cno")
             .print();
///dirtyHarry
        Utility.validate(outputFile, correct+"dirtyHarry.txt", false);
    }
    
}
