package Allegory.Family;

import PrologDB.DB;
import PrologDB.Table;
import PrologDB.ErrorReport;
import java.io.PrintStream;

public class Database {

    public familyTable family = familyTable.BaseRelation(this);
    public memberTable member = memberTable.BaseRelation(this);

    private DB db;

    public Database() {}

    public Database(String filename) {
        // Step 1: Read the database, throw exception if errors found and halt
        db = DB.read(filename);

        // Step 2: access MDELite tables
        Table _family = db.getTableEH("family");
        Table _member = db.getTableEH("member");

        // Step 3: pass1 -- load primitive data of all tables
        _family.tuplesLocal().forEach(t->family.makeAndAdd(t));
        _member.tuplesLocal().forEach(t->member.makeAndAdd(t));

        // Step 4: pass2 -- now replace MDELite tuple ids with Java object ids
        ErrorReport er = new ErrorReport();
        family.tuplesLocal().forEach(t->t.finishTuple(this,er));
        member.tuplesLocal().forEach(t->t.finishTuple(this,er));
        er.printEH(System.out);
    }

    public void print() { print(System.out); }

    public void prettyprint() { prettyprint(System.out); }

    public void prettyprint(PrintStream out) {
        out.format("dbase(Family,[family,member]).\n\n");
        family.prettyprintLocal(out);
        member.prettyprintLocal(out);
    }

    public void print(PrintStream out) {
        out.format("dbase(Family,[family,member]).\n\n");
        family.printLocal(out);
        member.printLocal(out);
    }

}
