package Allegory.startrek;

import PrologDB.DB;
import PrologDB.Table;
import PrologDB.ErrorReport;
import java.io.PrintStream;

//StartDB1
public class Database {

    public starshipTable starship = starshipTable.BaseRelation(this);
    public crewmanTable crewman = crewmanTable.BaseRelation(this);
    public commanderTable commander = commanderTable.BaseRelation(this);
    public lieutenantTable lieutenant = lieutenantTable.BaseRelation(this);

    private DB db;

    public Database() {}

    public Database(String filename) {
        // Step 1: Read the database, throw exception if errors found and hal
//StartDB1
        db = DB.read(filename);

        // Step 2: access MDELite tables
        Table _starship = db.getTableEH("starship");
        Table _crewman = db.getTableEH("crewman");
        Table _commander = db.getTableEH("commander");
        Table _lieutenant = db.getTableEH("lieutenant");

        // Step 3: pass1 -- load primitive data of all tables
        _starship.tuplesLocal().forEach(t->starship.makeAndAdd(t));
        _crewman.tuplesLocal().forEach(t->crewman.makeAndAdd(t));
        _commander.tuplesLocal().forEach(t->commander.makeAndAdd(t));
        _lieutenant.tuplesLocal().forEach(t->lieutenant.makeAndAdd(t));

        // Step 4: pass2 -- now replace MDELite tuple ids with Java object ids
        ErrorReport er = new ErrorReport();
        starship.tuplesLocal().forEach(t->t.finishTuple(this,er));
        crewman.tuplesLocal().forEach(t->t.finishTuple(this,er));
        commander.tuplesLocal().forEach(t->t.finishTuple(this,er));
        lieutenant.tuplesLocal().forEach(t->t.finishTuple(this,er));
        er.printEH(System.out);
    }

    public void print() { print(System.out); }

    public void prettyprint() { prettyprint(System.out);}

    public void prettyprint(PrintStream out) {
        out.format("dbase(startrek,[starship,crewman,commander,lieutenant]).\n\n");
        starship.prettyprintLocal(out);
        crewman.prettyprintLocal(out);
        commander.prettyprintLocal(out);
        lieutenant.prettyprintLocal(out);
        out.format("subtable(crewman,[commander,lieutenant]).\n");
//StartDB2
    }

    public void print(PrintStream out) {
        out.format("dbase(startrek,[starship,crewman,commander,lieutenant]).\n\n");
        starship.printLocal(out);
        crewman.printLocal(out);
        commander.printLocal(out);
        lieutenant.printLocal(out);
        out.format("subtable(crewman,[commander,lieutenant]).\n");
    }
//StartDB2
}
