package CD;

import java.io.PrintStream;
import java.util.LinkedList;

public class AssocSchema {

    static int cnt = 1;

    String class1, role1, class2, role2, name;
    atype at1, at2;
    int min1, max1, min2, max2;

    public static enum atype {
        DIAMOND(0, 1), BLACK_DIAMOND(1, 1), BLANK(0, -1), ONEPLUS(1,-1), CUSTOM(0, -1), UNKNOWN(0, 0);

        public final static int inf = 100;
        public final static String infs = "100";
        public final int min;
        public final int max;

        atype(int mn, int mx) {
            min = mn;
            max = mx;
        }

        public boolean geq(atype a) {
            switch (this) {
                case BLACK_DIAMOND:
                    return true;
                case DIAMOND:
                    return (a != BLACK_DIAMOND);
                case ONEPLUS: 
                    return (a != BLACK_DIAMOND && a != DIAMOND);
                case BLANK:
                    return (a != BLACK_DIAMOND && a != DIAMOND && a != ONEPLUS);
                case CUSTOM:
                    return a == CUSTOM;
                case UNKNOWN:
                    return a == UNKNOWN;
            }
            return false;
        }

        public int getMin() {
            return min;
        }

        public int getMax() {
            return (max == -1) ? inf : max;
        }

        public static class MinMax {

            boolean OK = false;
            int min = 0;
            int max = 0;

            public MinMax(String x) {
                if (x.contains(":")) {
                    String mm[] = x.split(":");
                    if (mm.length == 2) {
                        OK = true;
                        min = Integer.parseInt(mm[0]);
                        if (mm[1].equals("*") || mm[1].equals("-1")) {
                            max = -1;
                        } else {
                            max = Integer.parseInt(mm[1]);
                        }
                    }
                }
            }
        }

        public static atype convert(String x) {
            if (x.equals("")) {
                return BLANK;
            }
            for (atype a : values()) {
                if (x.equals(a.toString())) {
                    return a;
                }
            }
            throw new RuntimeException("Unknown atype (" + x + ")");
        }
    }
    // all except joinkey cannot be null

    public AssocSchema(String name, String class1, String role1, atype at1, String class2, String role2, atype at2) {
        this.name = name;
        this.class1 = class1;
        this.role1 = role1;
        this.at1 = at1;
        this.min1 = at1.min;
        this.max1 = at1.max;
        this.class2 = class2;
        this.role2 = role2;
        this.at2 = at2;
        this.min2 = at2.min;
        this.max2 = at2.max;
        patch(name, class1, role1, class2, role2);
    }

    public AssocSchema(String name, String class1, String role1, int min1, int max1,
            String class2, String role2, int min2, int max2) {
        this.name = name;
        this.class1 = class1;
        this.role1 = role1;
        this.at1 = atype.CUSTOM;
        this.min1 = min1;
        this.max1 = max1;
        if (max1 > atype.inf) {
            System.err.format("max1 value of %s exceeds %s", name, atype.inf);
        }
        this.class2 = class2;
        this.role2 = role2;
        this.at2 = atype.CUSTOM;
        this.min2 = min2;
        this.max2 = max2;
        if (max2 > atype.inf) {
            System.err.format("max2 value of %s exceeds %s", name, atype.inf);
        }
        patch(name, class1, role1, class2, role2);
    }

    private void patch(String name, String class1, String role1, String class2, String role2) {
        if (name.equals("")) {
            this.name = class1 + role1 + class2 + role2;
        }
        this.role1 = role1.equals("") ? "null" : role1;
        this.role2 = role2.equals("") ? "null" : role2;
    }

    // remember this is directed -- two associations will  be equal if
    // the have all the same values (except for id).
    public AssocSchema(LinkedList<String> parsed) {
//        int len = parsed.size();
//        if (len == 6 || len == 8) {  // if new syntax used
//            name = "l" + cnt++;
//            len++;
//        } else {                     // old syntax
            name = parsed.removeFirst();
//        }

        class1 = parsed.removeFirst();
        role1 = parsed.removeFirst();
        String next = parsed.removeFirst();
        atype x = atype.convert(next);
        if (x != atype.UNKNOWN) {
            at1 = x; 
            min1 = at1.min;
            max1 = at1.max;
        } else {
            at1 = atype.CUSTOM;
            min1 = Integer.parseInt(next);
            max1 = Integer.parseInt(parsed.removeFirst());
        }
        class2 = parsed.removeFirst();
        role2 = parsed.removeFirst();
        next = parsed.removeFirst();
        x = atype.convert(next);
        if (x != atype.UNKNOWN) {
            at2 = x;
            min2 = at2.min;
            max2 = at2.max;
        } else {
            at2 = atype.CUSTOM;
            min2 = Integer.parseInt(next);
            max2 = Integer.parseInt(parsed.removeFirst());
        }
        patch(name, class1, role1, class2, role2);
    }

    public String getClass1() {
        return class1;
    }

    public String getClass2() {
        return class2;
    }

    public String getRole1() {
        return role1;
    }

    public String getRole2() {
        return role2;
    }

    public atype getAt1() {
        return at1;
    }

    public int getMin1() {
        return min1;
    }

    public int getMax1() {
        return max1;
    }

    public atype getAt2() {
        return at2;
    }

    public int getMin2() {
        return min2;
    }

    public int getMax2() {
        return max2;
    }

    public String getName() {
        return name;
    }

    public boolean equals(AssocSchema other) {
        return name.equals(other.name) && at1.equals(other.at1)
                && at2.equals(other.at2)
                && class1.equals(other.class1) && class2.equals(other.class2)
                && role1.equals(other.role1) && role2.equals(other.role2);
    }

    public String toString() {
        if (at1==atype.CUSTOM || at2==atype.CUSTOM) {
            return String.format("assoc(%s,%s,%s,%d,%d,%s,%s,%d,%d).\n",
                name, class1, role1, min1, max1, class2, role2, min2, max2);
        } else
        return String.format("assoc(%s,%s,%s,%s,%s,%s,%s).\n",
                name, class1, role1, at1.toString(), class2, role2, at2.toString());
    }

    public void print(PrintStream out) {
        out.println(this);
    }

    public void print() {
        print(System.out);
    }
}
