package CD;

import MDLUtilities.Marquee1In_1Out;
import CD.AssocSchema.atype;
import PrologDB.DB;
import PrologDB.Table;
import PrologDB.Tuple;
import java.io.FileNotFoundException;

/**
 * unparses a cd specification specification into a plantuml spec
 */
public class CD2PlantUML extends MDLUtilities.RunningBear {
    static final String q = "\"";
    
    private static Table tab, attr, assoc, sub;

    /**
     * main prograqm
     *
     * @param args -- args
     */
    public static void main(String... args) {

        // Step 1: standard marquee processing
        Marquee1In_1Out mark = new Marquee1In_1Out(CD2PlantUML.class, ".cd.pl", ".class.plantuml", args);
        RBSetup(mark, args);

        cd2plantuml(db);
    }
    
    public static void cd2plantuml(DB db, String outputFileName) throws FileNotFoundException {
        openOut(outputFileName);
        cd2plantuml(db);
    }

    public static void cd2plantuml(DB db) {
        // step 1: open tables
        tab = db.getTableEH("tab");
        attr = db.getTableEH("attr");
        assoc = db.getTableEH("assoc");
        sub = db.getTableEH("sub");
        
        // Step 2: write header
        l("@startuml");
        
        // Step 3: output each table
        for (Tuple t: tab.tuples()) {
            Tuple supt;
            Table suptFields;
            
            supt = findSuperTable(t);
            if (supt!=null)
                suptFields = supt.rightSemiJoin("id", attr, "attrof");
            else 
                suptFields = new Table(attr);
            String ab = t.getBool("abs")?"abstract ":"";
            l("%sclass %s {",ab, t.get("name"));
            for (Tuple a : t.rightSemiJoin("id", attr, "attrof").tuples()) {
                String aname = a.get("name");
                if (aname.equals("id"))
                    continue;
                boolean inherited = suptFields.anyMatch(sf->sf.is("name",aname));
                if (inherited)
                    continue;
                String name2display = a.get("name");
                switch(a.get("ptype")) {
                    case "int":  
                    case "bool":
                    case "float":
                    case "dbl":
                        name2display = name2display + " : " + a.get("ptype");
                        break;
                    case "qstr":
                        name2display = q + name2display+q;
                        break;
                    case "str" :
                    default: 
                    
                }
                l("  %s", name2display);
            }
            l("}");
            l();
        }
        
        // Step 4: output each inheritance
        for (Tuple s : sub.tuples()) {
            String supname = tab.follow(s.get("suptab")).get("name");
            String subname = tab.follow(s.get("subtab")).get("name");
            l("%s <|-- %s",supname, subname);
        }
        
        // Step 4: output each association
        for (Tuple a : assoc.tuples()) {
            String left, right, connector;
            String class1name = tab.follow(a.get("class1")).get("name");
            String class2name = tab.follow(a.get("class2")).get("name");
            String role1 = a.get("role1");
            String role2 = a.get("role2");
            atype at = atype.convert(a.get("atype1"));
            switch (at) {
                case DIAMOND: 
                    left = role1;
                    right = role2 +card(a,'2');
                    connector = "o--";
                    break;
                case BLACK_DIAMOND:
                    left = role1;
                    right = role2 +card(a,'2');
                    connector = "*--";
                    break;
//                case NM:
                case BLANK:
                    left = role1;
                    right = role2;
                    connector = "--";
                    break;
                case CUSTOM:
                    left = role1 + card(a,'1');
                    right = role2 +card(a,'2');
                    connector = "--";
                    break;
                case UNKNOWN:
                default:  throw new RuntimeException("SHOULD NEVER GET HERE!");
            }
            l("%s \"%s\" %s \"%s\" %s",class1name,left, connector, right, class2name);
        }
            
        // Step X: finish
        l("@enduml");
        closeOut();
        
    }
    
    static String card(Tuple a, char whichone) { 
        String min = a.get("min"+whichone);
        String max = a.get("max"+whichone);
        if (min.equals("0") && max.equals("-1"))
            return "";
        if (min.equals("1")&& max.equals("-1"))
            return " 1+";
        if (min.equals("1") && max.equals("1"))
            return " 1";
        if (min.equals("0") && max.equals("1"))
            return " 0..1";
        return " " + min + ".." + max;
    }
    
    static Tuple findSuperTable(Tuple tb) {
        String tbid = tb.getId();
        for (Tuple t : sub.tuples()) {
            if (t.is("subtab",tbid)) 
                return tab.follow(t.get("suptab"));
        }
        return null;
    }
}