package CD;

import static CD.CD2Yuml.inputFileName;
import MDLUtilities.Marquee1In_1Out;
import PrologDB.Column;
import PrologDB.Column.opts;
import PrologDB.DB;
import PrologDB.DBSchema;
import PrologDB.OOSchema;
import PrologDB.SubTableSchema;
import PrologDB.Table;
import PrologDB.TableSchema;
import PrologDB.Tuple;
import java.util.LinkedList;

public class CD2Schema {
    static String strEnd = ":str";
    static String schemaEnd = ".schema.pl";

    static String outputFileName, appName;

    static Table tab, attr, assoc, sub;

    public static void main(String... args) {
        // Step 0 : standard marquee processing
        Marquee1In_1Out mark = new Marquee1In_1Out(CD2Yuml.class, ".cd.pl", schemaEnd, args);
        inputFileName = mark.getInputFileName();
        outputFileName = mark.getOutputFileName();
        appName = mark.getAppName();

        // Step 1: read in the input database
        DB db = DB.read(inputFileName);
        tab = db.getTableEH("tab");
        attr = db.getTableEH("attr");
        assoc = db.getTableEH("assoc");
        sub = db.getTableEH("sub");

        // Step 2: precondition -- every association must be a DIAMOND or BLACK_DIAMOND
        for (Tuple a : assoc.tuples()) {
            if (a.is("atype1", "BLACK_DIAMOND") || a.is("atype1", "DIAMOND")) {
                continue;
            }
            // manufacture a new association-class table
            
            String name = a.get("name");
            boolean duplicate = tab.select(t->t.is("name",name)).isNotEmpty();
            if (duplicate) {
                System.err.format("cannot create table for association (%s) as a table with that name already exists",name);
                return;
            }
            String classleft = a.get("class1");
            String classrite = a.get("class2");
            String roleleft  = a.get("role1");
            String rolerite  = a.get("role2");
            tab.add(name,name,"false");  // add non-abstract table
            attr.add(name+"_"+classleft,roleleft,classleft,name);
            attr.add(name+"_"+classrite,rolerite,classrite,name);
            
//            System.err.format("cannot translate CD file to schema: assoc(%s,%s...) does not have a BLACK_DIAMOND or DIAMOND end\n", a.getId(), a.get("name"));
//            return;
        }

        // Step 3: make (empty) tables
        OOSchema dbs = new OOSchema(appName);
        for (Tuple t : tab.tuples()) {
            // Step 3a: create empty table schema
            TableSchema ts = new TableSchema(t.get("name"));
            if (t.is("abs")) {
                ts.makeAbstract();
            }
            dbs.add(ts);
            String tid = t.getId();

            // Step 3b: now add attributes
            Column c;
            for (Tuple a : t.rightSemiJoin("id", attr, "attrof").tuples()) {
                //String name, opts opt, String kind, boolean isQuoted
                String ptype = a.get("ptype");
                boolean isq = ptype.equals("qstr");
                c = new Column(a.get("name"), opts.empty, ptype, isq);
                ts.add(c);
            }

            // Step 3c: now add attributes of associations
            for (Tuple lnk : assoc.tuples()) {
                if (lnk.is("class2", tid)) {
                    String c1name = getTableName(lnk.get("class1"));
                    if (lnk.is("atype1", "BLACK_DIAMOND")) {
                        c = new Column(lnk.get("role1"), opts.empty, c1name, false);
                        ts.add(c);
                    } else
                    if (lnk.is("atype1","DIAMOND")) {
                        c = new Column(lnk.get("role1"), opts.option, c1name, false);
                        ts.add(c);
                    }
                }
            }
        }

        // Step 4: add subtables
        SubTableSchema sts;
        TableSchema ts;
        for (Tuple s : sub.tuples()) {
            String supName = getTableName(s.get("suptab"));
            sts = dbs.getSubTableSchema(supName);
            if (sts == null) {
                ts = dbs.getTableSchemaEH(supName);
                sts = new SubTableSchema(ts);
                dbs.add(sts);
            }
            String subName = getTableName(s.get("subtab"));
            TableSchema sub = dbs.getTableSchemaEH(subName);
            sts.add(sub);
        }

        // Step 5: flatten and write write to disk
        
        dbs.addIdentifiersToAllNonSubtables();
        String schemaName = MDLUtilities.Utils.getAppName(outputFileName, schemaEnd);
//        String schemaName = outputFileName.replace(schemaEnd,"");
//        int last = schemaName.lastIndexOf("/");
//        if (last!=-1) {
//            schemaName = schemaName.substring(last+1);
//        }
        dbs.rename(schemaName);
        DBSchema dbf = dbs.finishAndPropagateAttributes();
        dbf.print(outputFileName);
//        System.out.format("%s produced\n", outputFileName);

    }

    static String getTableName(String id) {
        return tab.follow(id).get("name");
    }

}
