
package CD;

import MDLUtilities.Marquee1In_1Out;
import MDLUtilities.Utils;
import CD.AssocSchema.atype;
import PrologDB.DB;
import PrologDB.DBSchema;
import PrologDB.Table;
import PrologDB.Tuple;
import java.io.FileNotFoundException;
import java.io.PrintStream;


public class CD2Yuml {
    static String inputFileName, outputFileName, appName;
    static DB ydb;
    static Table tab, attr, assoc, sub, cons, ybox, yassoc; 
    
    static String temp = "temp.ypl.pl";
 
    public static void main(String... args) throws FileNotFoundException {
        // Step 0 : standard marquee processing
        Marquee1In_1Out mark = new Marquee1In_1Out(CD2Yuml.class, ".cd.pl", ".yuml.yuml", args);
        inputFileName = mark.getInputFileName();
        outputFileName = mark.getOutputFileName();
        appName = mark.getAppName();
        
        // Step 1: read in the input database
        DB db = DB.read(inputFileName);
        DB ydb = cd2ypl(db);
//        ydb.print(temp);
        
        // Step 8: convert temp into a .yuml.yuml file
        Yuml.ClassUnParser.ypl2Yuml(ydb,new PrintStream(outputFileName));
    }

    /**
     * @param db -- a .cd.pl database
     * @return ypl (yuml) database
     */
    public static DB cd2ypl(DB db) {
        tab = db.getTableEH("tab");
        attr = db.getTableEH("attr");
        assoc = db.getTableEH("assoc");
        sub = db.getTableEH("sub");
        cons = db.getTableEH("cons");
        
        // Step 2: create empty yuml database
        DBSchema dbs = DBSchema.read(Utils.MDELiteHome()+"libpl/ypl.schema.pl");
        ydb = new DB(appName,dbs);
        ybox = ydb.getTableEH("yumlBox");
        yassoc = ydb.getTableEH("yumlAssociation");
        
        // Step 3: translate tab + attr into boxes
        for (Tuple t : tab.tuples()) {
            Tuple n = new Tuple(ybox);
            String fields = makefields(t.getId());
            n.setValues(t.getId(),"c",t.get("name"),fields,"",t.get("abs"));
            ybox.add(n);
        }
        
        // Step 4: translate assoc into yassoc
        for (Tuple t : assoc.tuples()) {
            Tuple n = new Tuple(yassoc);
            String end1 = end(atype.convert(t.get("atype1")));
            String end2 = end(atype.convert(t.get("atype2")));
            n.setValues("a"+yassoc.newID(),t.get("class1"),t.get("role1"),end1,"-",
                    t.get("class2"),t.get("role2"),end2);
            yassoc.add(n);
        }
        
        // Step 5: translate inheritance into yassoc
        for (Tuple t : sub.tuples()) {
            Tuple n = new Tuple(yassoc);
            n.setValues("a"+yassoc.newID(),t.get("suptab"),"","^","-",
                    t.get("subtab"),"", "");
            yassoc.add(n);
        }
        
        for (Tuple t : cons.tuples()) {
            Tuple n = new Tuple(ybox);
            n.setValues("c"+cons.newID(),"n",t.get("name"),t.get("constraint"),"","false");
            ybox.add(n);
        }
        
        return ydb;
    }
    
    static String end(atype e) {
        switch(e) {
//            case NM: return "";
            case DIAMOND: return "<>";
            case BLACK_DIAMOND: return "++";
            default: return "";
        }
    }
    
    static String makefields(String tid) {
        String fields = "";
        String semicolon = "";
        for (Tuple a : attr.select(tt->tt.is("attrof",tid)).tuples()) {
            if (a.is("name","id")) 
                continue;
            fields = fields + semicolon + a.get("name");
            semicolon = ";";
        }
        return fields;
    }
}
