package CD;

import MDLUtilities.Marquee3In_1Out;
import MDLUtilities.RunningBear;
import PrologDB.DBSchema;
import PrologDB.TableSchema;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.nio.charset.StandardCharsets;
import java.util.LinkedList;

public class DBTranslation {

    static String schema1, schema2, txtJava, AppName, output;
    static LinkedList<TableSchema> tslist;

    public static void main(String... args)  {
        // Step 1: standard marquee 
        Marquee3In_1Out mark = new Marquee3In_1Out(DBTranslation.class, ".schema.pl", ".schema.pl", ".txt.jav", ".java", args);
        schema1 = mark.getInputFileName(1);
        schema2 = mark.getInputFileName(2);
        txtJava = mark.getInputFileName(3);
        AppName = mark.getAppName(txtJava, ".txt.jav");
        output = AppName + ".java";

        // Step 2: read in schemas
        DBSchema s1 = DBSchema.read(schema1);
        DBSchema s2 = DBSchema.read(schema2);
        //https://stackoverflow.com/questions/1760654/java-printstream-to-string
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        PrintStream ps = new PrintStream(os);
        s2.print(ps);
        String StrOfSchema;
        
        try {
        StrOfSchema = os.toString("UTF8").replace("\n", "!");
//            StringOfEntireSchema2 = StringOfEntireSchema2.substring(0,StringOfEntireSchema2.length()-1);
        StrOfSchema = StrOfSchema.replace("!", "\\n\"+\n\"");
        } catch (Exception e) {
            throw new RuntimeException("can't read file " + schema2);
        }

        // Step 3: create a list of tables that are identical in name
        // and sets of fields
        tslist = new LinkedList<>();
        for (TableSchema outer : s1.getTableSchemas()) {
            TableSchema inner = s2.getTableSchema(outer.getName());
            if (inner == null) {
                continue;
            }
            if (outer.equals(inner)) {
                tslist.add(outer);
            }
        }

        // Step 4: inhale custom code for translating changed tables
        String content;
        try {
            // Step 4: inhale custom code for copying old tables to new tablews
            //https://www.journaldev.com/875/java-read-file-to-string#java-read-file-to-string-using-apache-commons-io-fileutils-class
            content = org.apache.commons.io.FileUtils.readFileToString(new File(txtJava), StandardCharsets.UTF_8);
        } catch (IOException ex) {
            throw new RuntimeException("cannot read customized copy code of file" + txtJava + " " + ex.getMessage());
        }
//        content = content.replace("\"", "\\\"");

        // Step 5: generate the database-to-database translator
        codeGen.main(output);
        codeGen.header(mark, StrOfSchema);
        codeGen.customCode(content);
        codeGen.footer();

        // Step 6: done!
        System.out.format("translator %s produced!\n", output);
    }

    static class codeGen extends RunningBear {

        public static void main(String outputFile) {
            openOut(outputFile);
        }

        public static void header( Marquee3In_1Out mark, String StringOfEntireSchema2) {
            String s1 = mark.getAppName(schema1, ".schema.pl");
            String s2 = mark.getAppName(schema2, ".schema.pl");
            l("package DBXlate.DBTranslators;");
            l();
            l("import PrologDB.*;");
            l("import MDLUtilities.Marquee1In_1Out;");
            l();
            l("public class %s {", AppName);
            l();
            l("   static String StrOfTargetSchema = \"%s\";", StringOfEntireSchema2);
            l();
            makeCopyArray();
            l("   public static void main(String... args) {");
            l("      // Step 0: marquee processing");
            l("      Marquee1In_1Out mark = new Marquee1In_1Out(%s.class, \".%s.pl\", \".%s.pl\", args);",AppName, s1, s2);
            l("      String inputDBname = mark.getInputFileName();");
            l("      String outputDBname = mark.getOutputFileName();");
            l("      String AppName = mark.getAppName();");
            l();
            l("      DB idb = DB.read(inputDBname);");
            l("      DBSchema targetSchema = DBSchema.read(\"%s\",StrOfTargetSchema);", s2);
            l("      DB odb = new DB(AppName,targetSchema);");
            l();
            l("      Table otab, itab;");
            l("      for (String s : copy) {");
            l("         otab = odb.getTableEH(s);");
            l("         itab = idb.getTableEH(s);");
            l("         for (Tuple ti : itab.tuples()) {");
            l("            Tuple to = new Tuple(otab);");
            l("            to.copy(ti);");
            l("            otab.add(to);");
            l("         }");
            l("      }");
            l();
        }

        public static void makeCopyArray() {
            p("   static String copy[] = { ");
            for (TableSchema s : tslist) {
                p("\"%s\", ", s.getName());
            }
            l("};");
            l();
        }

        public static void customCode(String content) {
            l("%s", content);
            l();
        }

        public static void footer() {
            l("      odb.print(outputDBname);");
            l("      System.out.format(\" database %s translated to %s\\n\",inputDBname,outputDBname);", "%s", "%s");
            l("   }");
            l("}");
            l();
            closeOut();
        }
    }
}
