package CD;

import MDLUtilities.Marquee1In_1Out;
import MDLUtilities.Result;
import CD.AssocSchema.atype;
import PrologDB.DB;
import PrologDB.ErrorReport;
import PrologDB.SubTableSchema;
import PrologDB.Table;
import PrologDB.TableSchema;
import PrologDB.Tuple;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import static CD.AssocSchema.atype.*;

public class vpl2CDSpec {
    // don -- I think there is a vpl2Schema program in package Boot.allegory;
    // that is virtually identical.  Some maintenance is needed

    private static DB vplDb;
    private static CDSpec cds;
    private static Table vBox, vAsso;
    private static ErrorReport er;
    private static HashSet<String> ll; // for error reporting

    /**
     * Converts X.vpl.pl into X.cdschema.pl
     *
     * @param args -- single input file "X.vpl.pl"
     */
    public static void main(String... args) {
        // Step 1: standard marquee processing
        Marquee1In_1Out mark = new Marquee1In_1Out(vpl2CDSpec.class,
                ".vpl.pl", ".cd.pl", args);
        String inputFileName = mark.getInputFileName();
        String outputFileName = mark.getOutputFileName();
        String AppName = mark.getAppName();

        // Step 2: open the database and initialize error report
        //         create empty (output) schema
        vplDb = DB.read(inputFileName);
        process(vplDb).outputSchema(outputFileName);
    }

    public static Result process(DB vplDb) {
        vBox = vplDb.getTableEH("vBox");
        vAsso = vplDb.getTableEH("vAssociation");

        cds = new CDSpec(vplDb.getName());
        er = new ErrorReport();
        ll = new HashSet<>();
        
        // Step 3: create all tables 
        for (Tuple vb : vBox.tuples()) {
            String id = vb.get("id");
            String name = vb.get("name");

            // Step 3.0 prelim error checking
            if (!vb.is("methods", "")) {
                er.add("class %s should not have methods", name);
            }
            if (!vb.is("type", "c")) {
                er.add("only classes are permitted in this diagram: see box [%s,%s..}]",
                        id, name);
                continue;
            }

            // Step 3a: create attrstr table schema for it; check later if
            //          there is attrstr table with attrstr duplicate name
            //          also, add identifiers at the end.
            boolean isAbst = vb.getBool("abst");
            TableSchema tsch = new TableSchema(name);
            if (isAbst) {
                tsch.makeAbstract();
            }

            // Step 3b: add all attributes from class
            String attrstr = vb.get("fields");
            if (!attrstr.equals("")) {
                String[] atts = attrstr.split("%");
                for (String s : atts) {
                    if (!(s.equals("id") || s.startsWith("id:")))
                       tsch.add(s);
                }
            }
            cds.add(tsch);
        }

        // Step 4: now add association declarations
        // Step 3c: add attributes from links
        AssocSchema as;
        for (Tuple l : vAsso.tuples()) {
            if (l.is("arrow1", "TRIANGLE") || l.is("arrow2", "TRIANGLE")) {
                continue;
            }
            Tuple t1 = vBox.follow(l.get("cid1"));
            String c1name = t1.get("name");
            Tuple t2 = vBox.follow(l.get("cid2"));
            String c2name = t2.get("name");
            String role1 = l.get("role1");
            String role2 = l.get("role2");
            String aname = l.get("middleLabel");
            String arrow1 = l.get("arrow1");
            String arrow2 = l.get("arrow2");

            // move this into conform
            boolean ok = true;
            if (aname.equals("")) {
                if (role1.equals("") || role2.equals("")) {
                    er.add("too few role labels in association between classes (%s,%s)", c1name, c2name);
                }
                ok = false;
            }
            
            /// 
            if (arrow1.equals("BLACK_DIAMOND")) {
                makeAssoc(aname, c1name, role1, BLACK_DIAMOND, c2name, role2, atype.convert(arrow2));
                continue;
            }
            if (arrow2.equals("BLACK_DIAMOND")) {
                makeAssoc(aname, c2name, role2, BLACK_DIAMOND, c1name, role1, atype.convert(arrow1));
                continue;
            }
            if (arrow1.equals("DIAMOND")) {
                makeAssoc(aname,c1name, role1, DIAMOND, c2name, role2, atype.convert(arrow2));
                continue;
            }
            if (arrow2.equals("DIAMOND")) {
                makeAssoc(aname,c2name, role2, DIAMOND, c1name, role1, atype.convert(arrow1));
                continue;
            }

            // move this into conform
            if (aname.equals("")) {
                if (c1name.equals(c2name)) {
                    er.add("loop-association (%s,%s) must have a middle label", role1, role2);
                } else {
                    er.add("N:M-association (%s,%s) must have a middle label", role1, role2);
                }
                ok = false;
            }
            // and this too
            String[] R1 = role1.split(",");
            String[] R2 = role2.split(",");
            if (R1.length != R2.length) {
                er.add("replicated association between classes (%s,%s) has unbalanced number of roles", c1name, c2name);
                ok = false;
            }
            // and this!
            if (!ok) {
                continue;
            }

            makeAssoc(aname,c1name, role1, BLANK, c2name, role2, BLANK);
        }
        
        // Step 4: now create subtable hash map that contains the 
        //         data for creating subtable declarations
        HashMap<String, LinkedList<String>> subs = new HashMap<>();
        for (Tuple l : vAsso.tuples()) {
            String aid = l.get("id");
            String class1 = vBox.follow(l.get("cid1")).get("name");   //getClassName(l, "cid1");
            String class2 = vBox.follow(l.get("cid2")).get("name");   //getClassName(l, "cid2");
            if (l.is("arrow1", "TRIANGLE")) {
                if (subs.containsKey(class1)) {
                    LinkedList<String> sublist = subs.get(class1);
                    sublist.add(class2);
                    addAid(aid);
                    continue;
                } else {
                    LinkedList<String> sublist = new LinkedList<>();
                    sublist.add(class2);
                    subs.put(class1, sublist);
                    addAid(aid);
                    continue;
                }
            }
            if (l.is("arrow2", "TRIANGLE")) {
                if (subs.containsKey(class2)) {
                    LinkedList<String> sublist = subs.get(class2);
                    sublist.add(class1);
                    addAid(aid);
                    continue;
                } else {
                    LinkedList<String> sublist = new LinkedList<>();
                    sublist.add(class1);
                    subs.put(class2, sublist);
                    addAid(aid);
                    continue;
                }
            }
        }

        // Step 6: now create the subtable declarations
        for (Map.Entry<String, LinkedList<String>> entry
                : subs.entrySet()) {
            String key = entry.getKey();
            LinkedList<String> lst = entry.getValue();
            SubTableSchema sts = new SubTableSchema(cds.getTableSchemaEH(key));
            for (String c : lst) {
                TableSchema ts = cds.getTableSchema(c);
                if (ts == null) {
                    er.add("unknown table schema %s referenced", c);
                    continue;
                }
                sts.add(ts);
            }
            cds.add(sts);
        }

//        // Step 7: add identifiers to tables that are not subtables
//        cds.addIdentifiersToAllNonSubtables();

        // Step 9: write schema to output file
        er.printEH(System.out);
        return new Result(cds);
    }

    static String badAss = "association %s-%s is inconsistently replicated";

    static void makeAssoc(String name, String c1, String r1, AssocSchema.atype a1, String c2, String r2, AssocSchema.atype a2) {
        String[] R1 = r1.split(",");
        String[] R2 = r2.split(",");
        String[] M = name.split(",");
//        if (R1.length != R2.length || R2.length != M.length) {
        if (!(R1.length==R2.length && R2.length==M.length)) {
            er.add(badAss, c1, c2);
            return;
        }

        for (int i = 0; i < R1.length; i++) {
            AssocSchema as = new AssocSchema(M[i], c1, R1[i], a1, c2, R2[i], a2);
            cds.add(as);
        }

    }

    static String makeColumn(String name, String kind) {
        return name + ":" + kind;
    }

    static String getClassName(Tuple l, String whichCid) {
        Tuple t = l.rightSemiJoinFirst(whichCid, vBox, "id");
        if (t == null) {
            er.add("%s of vAssociation tuple with id=%s is non-existent",
                    whichCid, l.get("id"));
            return "WRONG";
        }
        return t.get("name");
    }

    static void addAid(String aid) {
        if (ll.contains(aid)) {
            er.add("association with id=%s has diamond and triangle", aid);
            return;
        }
        ll.add(aid);
    }
}
