package M4Parsing.PlantUMLPrims.Patterns;

import Parsing.GeneralPrimitives.Optionals.Choose1;
import Parsing.GeneralPrimitives.Optionals.Optional;
import Parsing.GeneralPrimitives.Token;
import Parsing.GeneralPrimitives.PrimTokens.End;
import Parsing.GeneralPrimitives.Patterns.Pattern;
import Parsing.GeneralPrimitives.PrimTokens.Name;
import Parsing.GeneralPrimitives.PrimTokens.QuotedString;
import Parsing.GeneralPrimitives.SaveTokens.Nothing;
import Parsing.LineParsers.LineToParse;
import M4Parsing.PlantUMLPrims.FixedStrTokens.Aggregation;
import M4Parsing.PlantUMLPrims.FixedStrTokens.Composition;
import M4Parsing.PlantUMLPrims.FixedStrTokens.Dashed;
import M4Parsing.PlantUMLPrims.FixedStrTokens.Dotted;
import M4Parsing.PlantUMLPrims.FixedStrTokens.Super;


/**
 * link PlantUML statement: NAME [quotedString] LeftTags Separator RightTags [quotedString] NAME [ ColonName ] end;
 *               LeftTags : LeftSuper | Association | Aggregation | Nothing;
 *               RightTags: RightSuper | Association | Aggregation | Nothing;
 *               ColonName: ":" NAME ;
 *               Separator: "--" | ".." ;
 */
public class LinkDecl extends Pattern {

    /** Standard constructor */
    public LinkDecl() {
        super();
    }

    /** Standard constructor with line to parse
     * @param l -- LineToParse
     */
    public LinkDecl(LineToParse l) {
        super(l);
    }
    
    @Override
    public Token[] getSequence() {
        if (sequence == null) {
            Token[] s = 
            {   new Name(), 
                new Optional( new QuotedString() ), 
                new Choose1( new Super(), new Composition(), new Aggregation(), new Nothing() ),
                new Choose1( new Dashed(), new Dotted()),
                new Choose1( new Super(), new Composition(), new Aggregation(), new Nothing() ),
                new Optional( new QuotedString() ),
                new Name(),
                new Optional( new ColonName() ),
                new End(),
            };
            sequence = s;
        }
        return sequence;
    }

    @Override
    public LinkDecl klone(LineToParse l) {
        return new LinkDecl(l);
    }
}
