package M4Utilities;

/** all run-time errors in MDELite */
public enum Error implements ErrInt<Error> {

    //badCommandLineArgs("incorrect command line arguments"),
    associationsDiffer("Association definitions in schemas NAME1 and NAME2 are different"),
    associationNoExist("Association ASSONAME does not exist"),
    cantCopy("Cannot copy from FROM to DESTINATION"),
    consSchemaNoExist("ConsSchema CONSNAME does not exist"),
    ioerror("In file FILENAME : ERROR"),
    noHome("M4 Home cannot be determined"),
    schemaFileNoExist("Schema file FILENAME does not exist or can't be opened"),
    wrongFileName("DB/Schema file FILENAME does not end in '.pl' or '.ooschema.pl'"),
    wrongCDName("CDSchema file FILENAME contains class diagram with name NAME");
    ;

    private final String msg;

    /** required constructor
     * 
     * @param msg -- string interpretation of error
     */
    Error(String msg) {
        this.msg = msg;
    }
    
    /** get error message of error */
    @Override
    public String getMsg() { 
        return msg;
    }
    
    /**
     * @return the enums of Error in an array
     */
    @Override
    public ErrInt<Error>[] vals() {
        return Error.values();
    }
    
    /** this is the constructor for throwing an error
     * 
     * @param error -- which error 
     * @param args -- arguments of error message
     * @return RuntimeException
     */
    public static RuntimeException toss(Error error, Object... args) {
//        if (error.msg.contains("FILENAME")) { args[0] = Utils.shortFileName((String) args[0]); }
        return new RuntimeException(MDLUtilities.Utils.makeString(error.msg,args));
    }  
}
