package Meta4;

import static MDLUtilities.RunningBear.l;
import PrologDB.OOSchema;
import PrologDB.SubTableSchema;
import PrologDB.TableSchema;

public class DatabaseFile extends MDLUtilities.RunningBear {

    static OOSchema db;

    public static void makeDatabaseFile(OOSchema s) {
        db = s;
        String filename = "Allegory/"+s.getName()+"/Database.java";
        openOut(filename);
        l("package Allegory.%s;", s.getName());
        l();
        l("import PrologDB.DB;");
        l("import PrologDB.Table;");
        l("import PrologDB.ErrorReport;");
        l("import java.io.PrintStream;");
        l();
        l("public class Database {");
        l();
        apply("    public %sTable %s = %sTable.BaseRelation(this);");
        l();
        l("    private DB db;");
        l();
        l("    public Database() {}");
        l();
        l("    public Database(String filename) {");
        l("        // Step 1: Read the database, throw exception if errors found and halt");
        l("        db = DB.read(filename);");
        l();
        l("        // Step 2: access MDELite tables");
        apply("        Table _%s = db.getTableEH(\"%s\");");
        l();
        l("        // Step 3: pass1 -- load primitive data of all tables");
        apply("        _%s.tuplesLocal().forEach(t->%s.makeAndAdd(t));");
        l();
        l("        // Step 4: pass2 -- now replace MDELite tuple ids with Java object ids");
        l("        ErrorReport er = new ErrorReport();");
        apply("        %s.tuplesLocal().forEach(t->t.finishTuple(this,er));");
        l("        er.printEH(System.out);");
        l("    }");
        printers(s);
        l("}");
        closeOut();
    }

    public static void apply(String pattern) {
        for (TableSchema ts : db.getTableSchemas()) {
            String tsname = ts.getName();
            l(pattern, tsname, tsname, tsname, tsname, tsname, tsname);
        }
    }
    
    public static void printers(OOSchema s) {
        l();
        l("    public void print() { print(System.out); }");
        l();
        l("    public void prettyprint() { prettyprint(System.out); }");
        l();
        pprint(s,true);
        pprint(s,false);
    }
    
    private static void pprint(OOSchema s, boolean prettyprint) {
        String prefix = prettyprint?"pretty":"";
        l("    public void %sprint(PrintStream out) {", prefix);
        p("        out.format(\"dbase(%s,[", s.getName());
        String comma = "";
        for (TableSchema ts : db.getTableSchemas()) {
            String tsname = ts.getName();
            p("%s%s",comma, tsname);
            comma = ",";
        }
        l("]).\\n\\n\");");
        for (TableSchema ts : db.getTableSchemas()) {
            String tsname = ts.getName();
            l("        %s.%sprintLocal(out);",tsname,prefix);
        }
        for (SubTableSchema sts : db.getSubTableSchemas()) {
            p("        out.format(\"subtable(%s,[", sts.getSuper().getName());
            comma = "";
            for (TableSchema ts : sts.getSubTableSchemas()) {
                p("%s%s",comma,ts.getName());
                comma = ",";
            }
            l("]).\\n\");");
        }
        l("    }");
        l();
    }
}
