package Meta4;

import CD.CDConform;
import CD.CDSpec;
import MDLUtilities.Marquee1In_1DirOut;
import MDLUtilities.Utils;
import PrologDB.DB;
import java.io.File;
import java.io.IOException;

public class Main {
    
    static final String outputDir = "Allegory";

    public static void main(String... args) throws IOException {

        // Step 1: standard marquee processing
        Marquee1In_1DirOut mark = new Marquee1In_1DirOut(Main.class, CDSpec.cdspecEnd, outputDir, args);
        String infile = mark.getInputFileName();
        String AppName = outputDir + "/"+ mark.getAppName();
        
        // Step 2: make sure that the outputDir is ok
        File od = new File(outputDir);
        if (od.exists()) {
            if (!od.isDirectory()) {
                System.out.format("%s is a file not a directory - cannot proceed\n", outputDir);
                System.exit(0);
            }
        } else {
            if (!od.mkdir()) {
                System.out.format("cannot create directory %s -- stop\n", outputDir);
                System.exit(0);
            }
        }

        // Step 3: prepare the directory
        try {
            Utils.deleteDirectory(AppName);
        } catch (Exception e) {
            System.out.println("cannot delete directory "+ AppName);
            System.exit(0);
        }
        if (!new File(AppName).mkdir()) {
            throw new RuntimeException("Could not create " + AppName + " directory");
        }

        // Step 4: read the schema into a useful form
        CDSpec s = CDSpec.read(infile);  
        DB db = s.toCD();
        db.print("tmp.cd.pl");
        CDConform.check(db);
        
        // Step 5: produce the files
      
        LangFile.makeLangFile(s);
        SchemaFile.addAssocFieldsAndTables(s); // this method modifies s
        DatabaseFile.makeDatabaseFile(s);
        TupleFiles.makeTupleFiles(s);
        TableFiles.makeTableFiles(s);
        SchemaFile.finishAndPrint(s);    // also modifies s
        

        
        System.out.println("output is in directory "+ AppName);
    }
    
    static void debug(CDSpec s, String dif) {
        System.out.format("---- start %s ----\n",dif);
        s.print();
        System.out.format("---- end   %s ----\n",dif);
    }

}

