package Meta4;

import MDLUtilities.RunningBear;
import static MDLUtilities.RunningBear.l;
import static MDLUtilities.RunningBear.p;
import PrologDB.Column;
import PrologDB.OOSchema;
import PrologDB.PrimType;
import PrologDB.SubTableSchema;
import PrologDB.TableSchema;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

public class PrintersAndUtilities extends RunningBear {
    
    public static void makeListOfSubSchemas( LinkedList<TableSchema> result, TableSchema ts, OOSchema s) {
        SubTableSchema sts = s.getSubTableSchema(ts);
        if (sts==null) 
            return;
        List<TableSchema> list = sts.getSubTableSchemas();
        if (list == null)
            return;
        for (TableSchema tsch : list) {
            result.add(tsch);
            makeListOfSubSchemas(result,tsch,s);
        }
    }

    public static Stack<TableSchema> makeStackOfSuperSchemas(TableSchema ts, OOSchema s) {
        Stack<TableSchema> result = new Stack<>();
        result.push(ts);
        while ((ts = getSuperSchema(ts, s)) != null) {
            result.push(ts);
        }
        return result;
    }

    public static TableSchema getSuperSchema(TableSchema ts, OOSchema s) {
        for (SubTableSchema sts : s.getSubTableSchemas()) {
            // find tableschema that is the parent of ts
            if (sts.contains(ts)) {
                ts = sts.getSuper();
                return ts;
            }
        }
        return null;
    }

    public static boolean hasNonPrimitiveColumns(TableSchema ts, OOSchema s) {
        Stack<TableSchema> stack = makeStackOfSuperSchemas(ts, s);
        while (!stack.isEmpty()) {
            ts = stack.pop();
            for (Column c : ts.getUnInheritedColumns()) {
                String okind = c.getKind();
                if (!PrimType.isPrim(okind)) {
                    return true;
                }
            }
        }
        return false;
    }

    public static void displayFieldsOO(OOSchema s, TableSchema ts, Printer printer) {
        Stack<TableSchema> stack = makeStackOfSuperSchemas(ts, s);
        while (!stack.isEmpty()) {
            ts = stack.pop();
            PrintersAndUtilities.displayFields(ts, printer);
        }
    }

    public static void displayFields(TableSchema ts, Printer pr) {
        for (Column c : ts.getUnInheritedColumns()) {
            String cname = c.getName();
            boolean isq = c.isQuoted();
            Column.opts opt = c.getOpt();
            String okind = c.getKind();
            boolean isPrim = PrimType.isPrim(okind);

            if (isPrim) {
                PrimType p = PrimType.convert2PrimType(okind);

                // Step 1b: crank out java definition for attributes
                switch (opt) {
                    case empty:
                    case option:
                        pr.printRealColumn(isq, p.javaName(), cname);
                    default:
                }
            } else { // pointers using foreign keys
                switch (opt) {
                    case empty:
                    case option:
//                        pr.printRealColumn(isq, okind, cname);
                        pr.printFakeColumn(isq, okind, cname);
                    case table:
                }
            }
            pr.commaUpdate();
        }
    }

    public static void displayAllFields(TableSchema ts, OOSchema s, Printer pr) {
        Stack<TableSchema> stack = makeStackOfSuperSchemas(ts, s);
        while (!stack.isEmpty()) {
            TableSchema TS = stack.pop();
            for (Column c : TS.getColumns()) {
                String cname = c.getName();
                boolean isq = c.isQuoted();
                Column.opts opt = c.getOpt();
                String okind = c.getKind();
                boolean isPrim = PrimType.isPrim(okind);

                if (isPrim) {
                    PrimType p = PrimType.convert2PrimType(okind);

                    // Step 1b: crank out java definition for attributes
                    switch (opt) {
                        case empty:
                        case option:
                            pr.printRealColumn(isq, p.javaName(), cname);
                        default:
                    }
                } else { // pointers using foreign keys
                    switch (opt) {
                        case empty:
                        case option:
//                        pr.printRealColumn(isq, okind, cname);
                            pr.printFakeColumn(isq, okind, cname);
                        case table:
                    }
                }
                pr.commaUpdate();
            }
        }
    }

    /*--------------------printer utilities-----------------*/
    public static interface Printer {

        void printRealColumn(boolean isQ, String kind, String cname);

        void printFakeColumn(boolean isQ, String kind, String cname);

        void commaUpdate();
    }

    static class fieldPrinter implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = "";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            if (!cname.equals("id")) {
                l("    public %s %s;", kind.replace("optional ", ""), cname);
            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            l("    public %s %s;", kind.replace("optional ", ""), cname);
            l("    protected String _%s;  // needed by database loader", cname);
        }
    }

    static class constPrinter implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = ", ";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            if (PrimType.isExtendedPrim(kind)) {
                p("%s%s %s", comma, kind, cname);
            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
        }
    }

    static class allConstArgsPrinter implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = ", ";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            p("%s%s %s", comma, kind.replace("optional ", ""), cname);
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            p("%s%s %s", comma, kind.replace("optional ", ""), cname);
        }
    }

    static class initAllFieldsPrinter implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = "";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            l("       this.%s = %s;", cname, cname);
//            if (PrimType.isExtendedPrim(kind)) {
//                l("       this.%s = %s;", cname, cname);
//            } else {
//                l("       this.%s = %s;", cname);
//            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            l("       this.%s = %s;", cname, cname);
        }
    }

    static class printPrinter implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = ",";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            p("%sa.%s", comma, cname);
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            p("%s__(a.%s)", comma, cname);
        }
    }

    static class initFieldsForConstructorPrinter implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = "";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
//            if (PrimType.isExtendedPrim(kind)) {
            l("       this.%s = %s;", cname, cname);
//            } else {
//                l("       this.%s = null;", cname);
//            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            l("       this.%s = null;", cname);
        }
    }

    static class initFieldsPrinter implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = "";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            switch (kind) {
                case "String":
                    l("       %s = t.get(\"%s\");", cname, cname);
                    break;
                case "boolean":
                    l("       %s = t.getBool(\"%s\");", cname, cname);
                    break;
                case "int":
                    l("       %s = t.getInt(\"%s\");", cname, cname);
                    break;
                case "float":
                    l("       %s = t.getFloat(\"%s\");", cname, cname);
                    break;
                case "double":
                    l("       %s = t.getDouble(\"%s\");", cname, cname);
                    break;
                default:
                    break;
            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            l("       _%s = t.get(\"%s\");", cname, cname);
        }
    }

    static class make1Printer implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = ", ";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            switch (kind) {
                case "String":
                    p("%st.get(\"%s\")", comma, cname);
                    break;
                case "boolean":
                    p("%st.getBool(\"%s\")", comma, cname);
                    break;
                case "int":
                    p("%st.getInt(\"%s\")", comma, cname);
                    break;
                case "float":
                    p("%st.getFloat(\"%s\")", comma, cname);
                    break;
                default:
                    break;
            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
        }
    }

    static class make2Printer implements Printer {

        @Override
        public void commaUpdate() {
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            l("       w._%s = t.get(\"%s\");", cname, cname);
        }
    }

    static class finishPrinter implements Printer {

        @Override
        public void commaUpdate() {
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            l("       if(!_%s.equals(\"null\"))", cname);
            l("           %s = db.%s.getTuple(_%s,er);", cname, kind.replace("optional ", ""), cname);
        }
    }

    static class gettersPrinter implements Printer {

        @Override
        public void commaUpdate() {
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            if (PrimType.isPrim(kind)) {
                l();
                l("    public %s %s() {", kind, cname);
                l("       return %s;", cname);
                l("    }");
            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
        }
    }

    static class getIdPrinter implements Printer {

        @Override
        public void commaUpdate() {
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            l();
            l("    @Override");
            l("    public %s getId() {", kind);
            l("       return %s;", cname);
            l("    }");
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
        }
    }

    static class toString1Printer implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = ",";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            if (isQ) {
                p("%s'%s'", comma, "%s");
            } else {
                p("%s%s", comma, "%s");
            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
        }
    }

    static class namesPrinter implements Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = ",";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
           if (PrimType.QStr.javaName().equals(kind)&& isQ)
               p("%sq(%s)", comma, cname);
           else
               p("%s%s", comma, cname);
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            p("%s__(%s)", comma, cname);
        }
    }

    static class make3Printer implements PrintersAndUtilities.Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = ",";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            switch (kind) {
                case "int":
                    p("%st.getInt(\"%s\")", comma, cname);
                    return;
                case "boolean":
                    p("%st.getBool(\"%s\")", comma, cname);
                    return;
                case "float":
                    p("%st.getFloat(\"%s\")", comma, cname);
                    return;
                case "double":
                    p("%st.getDouble(\"%s\")", comma, cname);
                    return;
                case "String":
                    p("%st.get(\"%s\")", comma, cname);
                default:
            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
        }
    }

    static class make4Printer implements PrintersAndUtilities.Printer {

        @Override
        public void commaUpdate() {
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            l("       r._%s = t.get(\"%s\");", cname, cname);
        }
    }

    static class formatPrinter implements PrintersAndUtilities.Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = ",";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            switch (kind) {
                case "int":
                    p("%s%s", comma, "%d");
                    return;
                case "boolean":
                    p("%s%s", comma, "%b");
                    return;
                case "float":
                    p("%s%s", comma, "%f");
                    return;
                case "double":
                    p("%s%s", comma, "%f");
                    return;
                default:
                case "String":
                    p("%s%s", comma, "%s");
                    return;
            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            p("%s%s", comma, "%s");
        }
    }

    static class make0Printer implements PrintersAndUtilities.Printer {

        @Override
        public void commaUpdate() {
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            p(",t.%s", cname);
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
        }
    }

    static class ColumnNamePrinter implements PrintersAndUtilities.Printer {

        String comma = "";

        @Override
        public void commaUpdate() {
            comma = ",";
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            String kkind = kind;
            switch (kind) {
                case "String":
                    kkind = "string";
                    break;
                default:
            }
            if (isQ) {
                p("%s'%s'", comma, cname);
                return;
            }
            if (kkind.equals("string")) {
                p("%s%s", comma, cname);
                return;
            }
            p("%s%s:%s", comma, cname, kkind);
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
            p("%s%s:%s",comma,cname,kind);
        }
    }

    static class ColumnValuePrinter implements PrintersAndUtilities.Printer {

        @Override
        public void commaUpdate() {
        }

        @Override
        public void printRealColumn(boolean isQ, String kind, String cname) {
            if (PrimType.isExtendedPrim(kind)) {
                p(",t.%s", cname);
            } else {
                p(",__(t.%s)", cname);
            }
        }

        @Override
        public void printFakeColumn(boolean isQ, String kind, String cname) {
        }
    }
}
