package Meta4;

import CD.CDSpec;
import MDLUtilities.RunningBear;
import static MDLUtilities.RunningBear.l;
import static MDLUtilities.RunningBear.openOut;
import static MDLUtilities.RunningBear.p;
import PrologDB.Column;
import static PrologDB.Column.opts.*;
import PrologDB.OOSchema;
import PrologDB.SubTableSchema;
import PrologDB.TableSchema;
import static Meta4.PrintersAndUtilities.*;
import CD.AssocSchema;
import PrologDB.PrimType;
import java.util.LinkedList;
import java.util.Stack;

public class TableFiles extends RunningBear {

    public static void makeTableFiles(CDSpec s) {
        for (TableSchema ts : s.getTableSchemas()) {
            String tname = ts.getName();
            String filename = String.format("Allegory/%s/%sTable.java", s.getName(), tname);
            openOut(filename);

            l("package Allegory.%s;", s.getName());
            l();
            l("import PrologDB.Tuple;");
            l("import java.io.PrintStream;");
            l("import java.util.LinkedList;");
            l("import Meta4Framework.TaBle;");
            l("import java.util.function.Function;");
            l("import static Allegory.%s.Lang.*;", s.getName());
            l();
            l("public class %sTable extends TaBle<Database,%s,%sTable> {", tname, tname, tname);
            l("    @Override");
            l("    public String TupleTypeName() { return \"%s\"; }", tname);
            l();
            l("    public <TABLE> TABLE apply%s(Function<%sTable,TABLE> f) { return f.apply(this); }",tname,tname);
            l();
            l("    // constructors for creating empty or singleton tables");
            l("    public %sTable(Database db) {", tname);
            l("        this(db,null);");
            l("    }");
            l();
            l("    public %sTable(Database db, %s t) {", tname, tname);
            l("        this.db = db;");
            l("        initTupleList(t);");
            l("    }");
            l();
            l("    // methods for loading a database");
            l("    static %sTable BaseRelation(Database db) {", tname);
            l("        %sTable tab = new %sTable(db);", tname, tname);
            l("        tab.computedTable = false;");
            l("        return tab;");
            l("    }");
            makeTuples(ts, s);
            l("    @Override");
            l("    public %sTable New() { return new %sTable(db); }", tname, tname);
            l();
            makeMethod(ts, s);
            deleteMethod(ts, s);
            cascadingDeleteMethod(ts, s);
            masthead(ts, s);
            printMethod(ts, s);
            getters(ts, s);
            displaySemiJoinMethods(ts, s);
            toTableMethods(ts,s);
            l("}");
            closeOut();
        }
    }

    private static void makeTuples(TableSchema ts, OOSchema s) {
        String tname = ts.getName();
        l();
        l("    @Override");
        l("    public LinkedList<%s> tuples() {", tname);
        SubTableSchema sts = s.getSubTableSchema(ts);
        if (sts == null) {
            l("        return tuples;");
        } else {
            l("        if (computedTable)");
            l("            return tuples;");
            l("        LinkedList<%s> all = new LinkedList<>();", tname);
            l("        all.addAll(tuples);");
            for (TableSchema xx : sts.getSubTableSchemas()) {
                l("        all.addAll(db.%s.tuples());", xx.getName());
            }
            l("        return all;");
        }
        l("    }");
        l();
    }

    private static void deleteMethod(TableSchema ts, OOSchema s) {
        l("    @Override");
        l("    public boolean deleteFromSubTables(%s t) {", ts.getName());
        LinkedList<TableSchema> result = new LinkedList<>();
        makeListOfSubSchemas(result, ts, s);
        if (!result.isEmpty()) {
            l("       if (computedTable) return false;");
            for (TableSchema t : result) {
                l("       if (t instanceof %s) return db.%s.delete((%s) t);", t.getName(), t.getName(), t.getName());
            }
        }
        l("       return false;");
        l("    }");
        l();
    }

    private static void cascadingDeleteMethod(TableSchema ts, CDSpec s) {
        String tname = ts.getName();
        l("    @Override");
        l("    public void cascadingDeleteLocal(%s t) {", tname);
        TableSchema sts = ts.getSuperTableOf();
        if (sts != null) {
            l("       db.%s.cascadingDeleteLocal((%s) t); // delete %s associations", sts.getName(), sts.getName(), sts.getName());
        }
        for (AssocSchema as : s.getAssocSchemas()) {
            // Step 1 : unpack and see if relevant
            String e1 = as.getClass1();
            String e2 = as.getClass2();

            if (!(e1.equals(tname) || e2.equals(tname))) {
                continue; // this assocschema is not relevant
            }
            String r1 = as.getRole1();
            String r2 = as.getRole2();
            String ml = as.getName();

            switch (as.getAt1()) {
                case BLACK_DIAMOND:
                    if (e1.equals(tname)) {
                        l("       t.%s().delete(); //b1", r2);
                    }
//                    if (e2.equals(tname)) {
//                        l("       t.select(x->x.%s==t).delete(); //b2", e1, r2);
//                    }
                    break;

                case DIAMOND:
                    if (e1.equals(tname)) {
                        l("       t.%s().forEach(x->x.%s=null); //b3", r2, r1);
                    }
//                    if (e2.equals(tname)) {
//                        l("       db.%s.select(x->x.%s==t).forEach(y->y.%s=null);  //b4", tname, r2, r2);
//                    }
                    break;
                case BLANK:
                case CUSTOM:
                    if (e1.equals(tname) && e2.equals(tname)) {
                        l("       db.%s.select(x->x.%s==t || x.%s==t).delete(); //b5", ml, r1, r2);
                    } else if (e1.equals(tname)) {
                        l("       db.%s.select(x->x.%s==t).delete(); //b6", ml, r1);
                    } else if (e2.equals(tname)) {
                        l("       db.%s.select(x->x.%s==t).delete(); //b7", ml, r2);
                    }
                    break;
                case UNKNOWN:
                    throw new RuntimeException("shouldn't reach here -- " + as.getAt1());
            }
        }
        l("    }");
        l();
    }

    // return dog(did,'name','breed',color).;"); 
    static void masthead(TableSchema ts, OOSchema s) {
        l("    @Override");
        l("    public void mastHead(PrintStream out) {");
        Column.opts o;
        p("        out.println(\"table(%s,[", ts.getName());
        displayFieldsOO(s, ts, new ColumnNamePrinter());
        l("]).\");");
        l("    }");
        l();
    }

    static void printMethod(TableSchema ts, OOSchema s) {
        TableSchema TS;
        int j;
        String comma;
        l("    @Override");
        l("    public String pformat() { return %s.fmt; }", ts.getName());
        l();
        l("    @Override");
        l("    public String ppformat(boolean local) {");
        p("       int ");
        Stack<TableSchema> stack = makeStackOfSuperSchemas(ts, s);
        j = 0;
        comma = "";
        while (!stack.isEmpty()) {
            TS = stack.pop();
            for (Column c : TS.getColumns()) {
                p("%s f%d=0", comma, j++);
                comma = ",";
            }
        }
        l(";");
        l("       for (%s a : local?tuples:tuples()) {", ts.getName());
        stack = makeStackOfSuperSchemas(ts, s);
        j = 0;
        while (!stack.isEmpty()) {
            TS = stack.pop();
            for (Column c : TS.getColumns()) {
                if (j == 0 || c.getKind().equals("String")) {
                    l("          f%d = Math.max(f%d,a.%s.length());", j, j++, c.getName());
                } else if (!PrimType.isPrim(c.getKind())) {
                    l("          f%d = Math.max(f%d,a.%s.id.length());", j, j++, c.getName());
                } else {
                    l("          f%d = Math.max(f%d,(\"\"+a.%s).length()+2);", j, j++, c.getName());

                }
            }
        }
        l("       }");
        p("       return String.format(\"%s(", ts.getName());

        j = 0;
        comma = "";
        String rest = "";
        stack = makeStackOfSuperSchemas(ts, s);
        while (!stack.isEmpty()) {
            TS = stack.pop();
            for (Column k : TS.getColumns()) {
                PrimType p = PrimType.convertJavaName2PrimType(k.getKind());
                if (p != null) {
                    p("%s%s%c", comma, "!%d", p.javaformat());
                    rest = rest + ",f" + j++;
                } else {
                    p("%s%ss", comma, "!%d");
                    rest = rest + ",f" + j++;
                }
                comma = ",";
            }
        }
        l(").\\n\"%s).replace(\"!\",\"%c\");", rest, '%');
        l("    }");
        l();
    }

    public static void makeMethod(TableSchema ts, OOSchema s) {
        String name = ts.getName();
        l("    @Override");
        l("    public void makeAndAdd(Tuple t) {", name);
        p("       %s r = new %s(", name, name, name);
        displayFieldsOO(s, ts, new make3Printer());
        l(");");
        displayFieldsOO(s, ts, new make4Printer());
        l("       add(r);");
        l("    }");
        l();
    }

    static void getters(TableSchema ts, OOSchema s) {
        Stack<TableSchema> stack = makeStackOfSuperSchemas(ts, s);
        while (!stack.isEmpty()) {
            TableSchema TS = stack.pop();
            for (Column c : TS.getColumns()) {
                String cname = c.getName();
                Column.opts o = c.getOpt();
                String kind = c.getKind();
                if (o == table || !PrimType.isPrim(kind)) {
                    continue;
                }

                PrimType p = PrimType.convert2PrimType(kind);
                String ckind = p.langName();
                l("    public %sTable %s() {", ckind, cname);
                l("       %sTable st = new %sTable(db);", ckind, ckind);
                l("       tuples.forEach(t -> st.add(new %s(t.%s)));", ckind, cname);
                l("       return st;");
                l("    }");
                l();
            }
        }
    }

    static String firstCharIsUpper(String str) {
        return Character.toUpperCase(str.charAt(0)) + str.substring(1);
    }

    public static void displaySemiJoinMethods(TableSchema ts, CDSpec s) {
        String tname = ts.getName();
        for (AssocSchema as : s.getAssocSchemas()) {
            // Step 1 : unpack and see if relevant
            String e1 = as.getClass1();
            String e2 = as.getClass2();

            if ((s.isGEQ(e1, tname) || s.isGEQ(e2, tname)) == false) {
                continue; // this assocschema is not relevant
            }
            String r1 = as.getRole1();
            String r2 = as.getRole2();
            String ml = as.getName();

            // Step 2: deal with nulls
            r1 = r1.equals("null") ? ml : r1;
            r2 = r2.equals("null") ? ml : r2;

            switch (as.getAt1()) {
                case BLACK_DIAMOND:
////                    if (tname.equals(e1)) {
                    if (s.isGEQ(e1, tname)) {
                        diamondRight(e2, r2, tname, r1, false);
                    }
                    if (s.isGEQ(e2, tname)) {
                        diamondLeft(e1, r1, tname, r2, false);
                    }
                    break;

                case DIAMOND:
////                    if (tname.equals(e1)) {
                    if (s.isGEQ(e1, tname)) {
                        diamondRight(e2, r2, e1, r1, true);
                    }
                    if (s.isGEQ(e2, tname)) {
                        diamondLeft(e1, r1, e2, r2, true);
                    }
                    break;
//                case NM:
                case BLANK:
////                    if (tname.equals(e1)) {
                    if (s.isGEQ(e1, tname)) {
                        nmLeft(e2, r2, tname, ml, r1);
                    }
                    if (s.isGEQ(e2, tname)) {
                        nmLeft(e1, r1, tname, ml, r2);
                    }
                    break;
            }
        }
    }

    public static void nmLeft(String e2, String r2, String e1, String ml, String r1) {
        l("    public %sTable %s() {", e2, r2);
        l("       LinkedList<%s> tups = tuples();", e1);
        l("       %sTable result = new %sTable(db);", e2, e2);
        l("       for(%s r : db.%s.tuples()) {", ml, ml);
        l("          if(tups.contains(r.%s) && !result.contains(r.%s)) {", r1, r2);
        l("             result.add(r.%s);", r2);
        l("          }");
        l("       }");
        l("       return result; // nmLeft");
        l("    }");
        l();
    }

    public static void diamondLeft(String e1, String r1, String e2, String r2, boolean testNull) {
        l("    public %sTable %s() {", e1, r1);
        l("       %sTable result = new %sTable(db);", e1, e1);
        l("       for (%s w : tuples()) {", e2, e2);
        if (testNull) {
            l("          if (w.%s==null) continue;", r1);
        }
        l("          if (!result.contains(w.%s))", r1);
        l("              result.add(w.%s);", r1);
        l("       }");
        l("       return result; // diamondLeft", e1, r2);
        l("    }");
        l();
    }

    public static void diamondRight(String e2, String r2, String e1, String r1, boolean testNull) {
        l("    public %sTable %s() {", e2, r2);
        l("       LinkedList<%s> tups = tuples();", e1);
        l("       %sTable result = new %sTable(db);", e2, e2);
        l("       for(%s a : db.%s.tuples()) {", e2, e2);
        if (testNull) {
            l("          if(a.%s==null) continue;", r1);
        }
        l("          if(tups.contains(a.%s) && !result.contains(a)) {", r1);
        l("             result.add(a);");
        l("          }");
        l("       }");
        l("       return result; // diamondRight");
        l("   }");
        l();
    }
    
    public static void toTableMethods(TableSchema ts, OOSchema s) {
        // Step 1: create a toTable() method for each supertable
        //         no need to check instanceof -- it is already satisfied
        l("   /* the following methods cast a subtable into a supertable */");
        l();
        Stack<TableSchema> result = makeStackOfSuperSchemas(ts,s);
        while (!result.isEmpty()) {
            TableSchema target = result.pop();
            if (target == ts) 
                continue;
            String tname = target.getName();
            String Tname = tname.substring(0,1).toUpperCase() + tname.substring(1);
            l("   public %sTable to%sTable() {",tname, Tname);
            l("      %sTable tb = new %sTable(db);",tname, tname);
            l("      for (%s t : tuples()) { tb.add((%s) t); }", tname,tname);
            l("      return tb;");
            l("   }");
            l();
        }
        
        // Step 2: create a toTable() method for each subtable
        //         must check instanceof, as supertable has tuples of all
        //         subtables
        l("   /* the following methods filter a supertable into a subtable */");
        l();
        LinkedList<TableSchema> tlist = new LinkedList<>();
        String tsname = ts.getName();
        makeListOfSubSchemas(tlist,ts,s);
        for(TableSchema x : tlist) {
            String tname = x.getName();
            String Tname = tname.substring(0,1).toUpperCase() + tname.substring(1);
            l("   public %sTable to%sTable() {",tname, Tname);
            l("      %sTable tb = new %sTable(db);",tname, tname);
            l("      for (%s t : tuples()){ if (t instanceof %s) tb.add((%s) t); }",tsname,tname,tname);
            l("      return tb;");
            l("   }");
            l();
        }
        l("   /* end of filtering methods */");
        l();
    }
}
