package Meta4Framework.O;

import java.io.PrintStream;
import java.util.LinkedList;

public class OTable {

    String name;
    public LinkedList<OTuple> tuples = new LinkedList<>();
    OCol[] cols;

    public OTable(String name, OCol... cols) {
        this.name = name;
        this.cols = cols;
    }
    
    public void clear() {
        tuples.clear();
    }
    
    public OCol[] getColumns() { 
        return cols;
    }

    public void add(OTuple t) {
        if (t.size() != cols.length) {
            String msg = "OTuple size (" + t.size() + ") does not match OTable size (" + cols.length + ")";
            throw new RuntimeException(msg);
        }
        tuples.add(t);
        t.belongsTo = this;
    }
    
    public void add(Object...vals) {
        OTuple t = new OTuple(vals);
        add(t);
    }
    
    public void collect(OTable tbl)  {
        for (OTuple ot : this.tuples) {
            tbl.add(ot);
        }
    }

    @Override
    public String toString() {
        String result = "table(" + name + ",[";
        String comma = "";
        for (OCol oc : cols) {
            switch(oc.getType()) {
                case Str:
                    result = String.format("%s%s%s", result, comma, oc.name);
                    break;
                case QStr:
                    result = String.format("%s%s\"%s\"", result, comma, oc.name);
                    break;
                default: 
                   result = String.format("%s%s%s:%s", result, comma, oc.name, oc.getType());
            }
            comma = ",";
        }
        result = result + "]).\n";
        for (OTuple ot : tuples) {
            result += ot.toString();
        }
        return result + "\n";
    }

    public void print(PrintStream out) {
        out.print(this);
    }

    public void print() {
        System.out.print(this);
    }
}
