package Allegory.Person;

import PrologDB.DB;
import PrologDB.Table;
import PrologDB.ErrorReport;
import java.io.PrintStream;

public class Database {

    public personTable person = personTable.BaseRelation(this);
    public maleTable male = maleTable.BaseRelation(this);
    public femaleTable female = femaleTable.BaseRelation(this);

    private DB db;

    public Database() {}

    public Database(String filename) {
        // Step 1: Read the database, throw exception if errors found and halt
        db = DB.read(filename);

        // Step 2: access MDELite tables
        Table _person = db.getTableEH("person");
        Table _male = db.getTableEH("male");
        Table _female = db.getTableEH("female");

        // Step 3: pass1 -- load primitive data of all tables
        _person.tuplesLocal().forEach(t->person.makeAndAdd(t));
        _male.tuplesLocal().forEach(t->male.makeAndAdd(t));
        _female.tuplesLocal().forEach(t->female.makeAndAdd(t));

        // Step 4: pass2 -- now replace MDELite tuple ids with Java object ids
        ErrorReport er = new ErrorReport();
        person.tuplesLocal().forEach(t->t.finishTuple(this,er));
        male.tuplesLocal().forEach(t->t.finishTuple(this,er));
        female.tuplesLocal().forEach(t->t.finishTuple(this,er));
        er.printEH(System.out);
    }

    public void print() { print(System.out); }

    public void prettyprint() { prettyprint(System.out); }

    public void prettyprint(PrintStream out) {
        out.format("dbase(Person,[person,male,female]).\n\n");
        person.prettyprintLocal(out);
        male.prettyprintLocal(out);
        female.prettyprintLocal(out);
        out.format("subtable(person,[male,female]).\n");
    }

    public void print(PrintStream out) {
        out.format("dbase(Person,[person,male,female]).\n\n");
        person.printLocal(out);
        male.printLocal(out);
        female.printLocal(out);
        out.format("subtable(person,[male,female]).\n");
    }

}
