package Drivers.Alleg;

import Allegory.Alleg.Database;
import Allegory.Alleg.Dept;
import Allegory.Alleg.DeptTable;
import Allegory.Alleg.EmpTable;
import static Allegory.Alleg.Lang.*;
import MDLUtilities.Marquee1In_1Out;
import PrologDB.ErrorReport;
import java.io.PrintStream;
import java.util.OptionalInt;

public class Driver {

    public static void main() throws Exception {
        String[] args = { "test/Allegory/Alleg/TestData/x.alleg.pl" };
        main(args);
    }
    
    static PrintStream outt; 
    public static void main(String... args) throws Exception {
        // Standard marquee processing
        Marquee1In_1Out mark = new Marquee1In_1Out(Driver.class, ".alleg.pl",".txt", args);
        String inputFileName = mark.getInputFileName();
        String outputFileName = mark.getOutputFileName();
        Database db = new Database(inputFileName);
        outt = new PrintStream(outputFileName);
        outt.println("THE START");

//        boolean r = db.Person.reject(p->p.id()==p).exists(true);
//        outt.println("1) id arrow test "+r?"works!":"fails!");
        outt.println("2) print all persons whose name begins with 'd' or 'p'");
///two
        db.Emp.select(t -> t.name.startsWith("d") || t.name.startsWith("p")).print(outt);
///two        
//        out.println("all persons who work in division v1");
//        division.select(d->d.is("vid","v1")).hasDeps().employs().print();

        outt.println("3) print all divisions in which don works");
///three
        db.Emp.select(t -> t.name.equals("don")).worksin().inDiv().print(outt);
///three

        
        outt.println("4) print all persons that work in the same department as priscila");
///four
        db.Emp.select(t -> t.name.equals("priscila")).worksin().employs().print(outt);
///four

        
        outt.println("5) who are the division colleagues of priscila");
///five
        db.Emp.select(t -> t.name.equals("priscila")).worksin().inDiv().hasDept().employs().print(outt);
///five

        outt.println("6) print the children of employees that work in this company");
///six
        db.Emp.chldOf().print(outt);
///six
        //Step 7: are there children who work in the same department as their parent?
///seven
        outt.println("7) what children work in the same department as their parent?\n");
        db.Emp.forEach(p -> {
            DeptTable pdepts = p.worksin();
            EmpTable child = p.chldOf();
            DeptTable cdepts = child.worksin();
            DeptTable result = pdepts.intersect(cdepts);
            if (result.exists()) {
                outt.format("%s and 1+ child work in the same department\n", p.name);
                result.print(outt);
            }
        });
///seven



//        // 8: lets do some joins to see if AllegoryCommon works
//        //    print for each department, print the name of division that it is in
//        out.println("8) for each department, print the name of division that it is in?\n");
//        department.join("inDiv", division, "id")
//                .project("Department.name","Division.name").print();
        // test group by
        outt.println();
        outt.println("group by city\n");
        db.Dept.groupBy(t -> t.city, tbl -> display(tbl));
       
        
        outt.println();
        outt.println("group by city and aggregate\n");
        db.Dept.groupBy(t -> t.city, tbl -> displayAgg(tbl));
      
        outt.println();
        outt.println("find divisions that have departments in Austin");
        db.Div.select(d->d.hasDept().select(r->r.city.equals("Austin")).exists())
                .print(outt);
        
        outt.println();
        outt.println("test of link2 operation: list departments in goods division");
        db.Dept.select(d->d.link2inDiv().name.equals("goods")).print(outt);
        
        // Step 10: 	list the ID of each employee (whose parent is an employee) 
        // with the ID of division(s) in which he/she works.
        
//        out.println("10: list the ID of each employee (whose parent");
//        out.println("    is an employee) with the ID of the division(s)");
//        out.println("     in which he/she works");
//        
//        db.Emp.select(e->e.parOf().exists()).
        /****************** error reporting tests ******************/
        
        outt.println("Constraints");
        outt.println();
        ErrorReport er = new ErrorReport();

        outt.println("All emps have unique names\n");
        outt.println("Solution 1");
        db.Emp.name()
              .duplicates()
              .error(er,"several Emp tuples have name '%s'",e->e.value);
        er.print(outt);
        er.clear();
        outt.println("Solution 2");
        STRINGTable dups = db.Emp.name().duplicates();
        db.Emp.select(e->dups.contains(e.name))
                .error(er,"Emp tuple with id=%s has non-unique name '%s'",r->r.id, r->r.name);
//        out.println("way 1 after duplicate added");
//        db.Emp.add(new Emp("p7","don",40));
//        db.Emp.name().duplicates().error(er,"multiple Emp tuples have name '%s'",e->e.value);
        er.print(outt);
        er.clear();

//        out.println("way 2 after duplicate added");
//        Lang.STRINGTable dups2 = db.Emp.name().duplicates();
//        db.Emp.select(e->dups2.contains(e.name)).error(er,"Emp tuple with id=%s has non-unique name '%s'",r->r.id, r->r.name);
//        er.print(out);
        

        outt.println("\nfor each department print its employees");
        DTable dt = new DTable("depair","dname", "ename", "eage");
        db.Dept.forEach(d->d.employs().forEach(e->dt.add(d.name,e.name,e.age+"")));
        dt.print(outt);
        er.clear();
        outt.println("\ndept in toronto cannot hire emps younger that 19");
        
outt.println("Done");
   if (true) return;
//        DepartmentTable xxx=  db.Department
//          .select(f->f.city.equals("Toronto"));
//        xxx.print();
//        xxx.forEach(d->d.employs()
//                       .forEach(e-> { if (e.age<19)
//                                         er.add("%s illegally hired %s",d.name,e.name); }));
        
        db.Dept
                .select(d->d.city.equals("Toronto"))
                .forEach(d->d.employs().select(ee->ee.age<19)
                        .error(er,"%s illegally hired %s",e->d.name,e->e.name));
        er.print(outt);
        er.clear();
        
        outt.println("no division can employ more than 20 workers");
        db.Div
                .select(d->d.hasDept().employs().count()>20)
                .error(er, "%s has too many workers", d->d.name);
        er.print(outt);
        outt.println("THE END");
        outt.close();
    }

    public static void display(DeptTable t) {
        Dept d = t.getFirst();
        outt.format("%10s group\n", d.city);
        t.print(outt);
    }
    
    public static void displayAgg(DeptTable t) {
        Dept d = t.getFirst();
        outt.format("%10s has %d department(s)\n", d.city, t.count());
    }
    
    public static void Query(String... args) throws Exception {
//        String[] args = { "test/Allegory/Alleg/TestData/x.alleg.pl", "query.txt" };
        Marquee1In_1Out mark = new Marquee1In_1Out(Driver.class, ".alleg.pl","*.txt", args);
        String inputFileName = mark.getInputFileName();
        String outputFileName = mark.getOutputFileName();
        Database db = new Database(inputFileName);
        PrintStream out = new PrintStream(outputFileName);

        out.println("(Q1) Find all employees whose name");
        out.println("     begins with \"d\" or \"p\"\n");
        db.Emp.select(t -> t.name.startsWith("d") || t.name.startsWith("p")).print(out);
        out.println();

        out.println("(Q2) Find all divisions that have");
        out.println("     departments in Austin\n");
        db.Div.select(d->d.hasDept().select(r->r.city.equals("Austin")).exists()).print(out);
        out.println();

        out.println("old(Q3)  Find the children of employees ");
        out.println("      that are also employees\n");
        db.Emp.chldOf().print(out);

        out.println("old(Q4) List the employees in the goods division that work in Hamilton");
        db.Div.select(d->d.name.equals("goods"))
          .hasDept().select(p->p.city.equals("Hamilton"))
          .employs().print(out);
        out.println();
        
        
        out.println("(Q3) List employees that work in multiple departments");
        db.Emp.select(e->e.worksin().count()>1).print(out);
        out.println();
        
        out.println("(Q4)  Find all divisions in which don works\n");
        db.Emp.select(t -> t.name.equals("don")).worksin().inDiv().print(out);


        out.println("(Q5)  Find the division colleagues of priscila\n");
        db.Emp.select(t -> t.name.equals("priscila")).worksin().inDiv().hasDept().employs().print(out);
        
        out.println("old(Q6)  Find children that work in the");
        out.println("      same department as their parent\n");
        db.Emp.forEach(p -> {
            DeptTable pdepts = p.worksin();
            EmpTable child = p.chldOf();
            DeptTable cdepts = child.worksin();
            DeptTable result = pdepts.intersect(cdepts);
            if (result.exists()) {
                out.format("%s and 1+ child work in the same department\n", p.name);
                result.print(out);
            }
        });
                
        out.println("(Q6) list the ID of each employee (whose parent is an employee)");
        out.println("     with the ID of division(s) in which he/she works.");
        DTable q6 = new DTable("q6","emp.id","div.id");
        db.Emp.select(e->e.parOf().exists())
                .forEach(em->em.worksin().inDiv().forEach(d->q6.add(em.id,d.id)));
        out.print(q6);
        
        out.println("");
        out.println("(Old) list name of employee and the names departments in which he/she and at least one child work");
        db.Emp.forEach(p -> {
            DeptTable result = p.worksin().intersect(p.chldOf().worksin());
            if (result.exists()) {
                out.format("%s in ", p.name);
                result.forEach(d->out.format("%s ",d.name));
                out.format("\n");
            }
        });
        
        out.println("");
        out.println("(Old) list name of employee and the names departments in which he/she and at least one child work");
        DTable q5 = new DTable("q5","parentName", "DeptName");
        db.Emp.forEach(p -> {
            DeptTable result = p.worksin().intersect(p.chldOf().worksin());
            result.forEach(d->q5.add(p.name,d.name));
        });
        out.print(q5);

        
        out.println("");
        out.println("(Q7) What is the most number of employees in any department?");
        OptionalInt m = db.Dept.stream().mapToInt(d->d.employs().count()).max();
        if(m.isPresent()) out.println("Ans: "+m.getAsInt());
        
        //---- done
        out.close();
    }
    
    public static void Constraints(String... args) throws Exception {
//        String[] args = { "test/Allegory/Alleg/TestData/x.alleg.pl", "trash/out.txt" };
        Marquee1In_1Out mark = new Marquee1In_1Out(Driver.class, ".alleg.pl", ".txt", args);
        String inputFileName = mark.getInputFileName();
        String outputFileName = mark.getOutputFileName();
        Database db = new Database(inputFileName);
        PrintStream out = new PrintStream(outputFileName);
        ErrorReport er = new ErrorReport();

        out.println("(C1) All Emps must have unique names\n");
        out.println("Solution 1: ");
        db.Emp.name().duplicates().error(er,"several Emp tuples have name '%s'",e->e.value);
        er.print(out);
        er.clear();
        out.println("Solution 2:");
        STRINGTable dups = db.Emp.name().duplicates();
        db.Emp.select(e->dups.contains(e.name))
                .error(er,"Emp tuple with id=%s has non-unique name '%s'",r->r.id, r->r.name);
//        out.println("way 1 after duplicate added");
//        db.Emp.add(new Emp("p7","don",40));
//        db.Emp.name().duplicates().error(er,"multiple Emp tuples have name '%s'",e->e.value);
        er.print(out);
        er.clear();
        
        out.println("(C2) All Depts in Toronto cannot");
        out.println("     employ Emps younger than 19\n");
                db.Dept
                .select(d->d.city.equals("Toronto"))
                .forEach(d->d.employs().select(ee->ee.age<19)
                        .error(er,"%s illegally hired %s",e->d.name,e->e.name));
        er.print(out);
        er.clear();   
           
        out.println("(C3) No Div can employ more than 20 Emps\n");
        db.Div
                .select(d->d.hasDept().employs().count()>20)
                .error(er, "%s has too many workers", d->d.name);
        er.print(out);
        er.clear();
        
        out.println("(C4) No more than two generations of workers can be employed\n");
        db.Emp.select(e->e.chldOf().chldOf().exists())
              .error(er, "%s has >2 generations of family members employed", e->e.name);
        er.print(out);
        er.clear();
        
        out.println("old (C4) Which children have >1 parents employed\n");
        db.Emp.select(e->e.parOf().count()==2)
              .error(er, "%s has >1 parents employed", e->e.name);
        er.print(out);
        er.clear();
        
    }     
        
}
